#include "StdAfx.h"
#include "GTCamera.h"
#include "algebra.h"

GTCamera::GTCamera(void)
: BrightFrame(NULL)
{
	DarkFrame = NULL;
	BrightFrame = NULL;
	work = NULL;
	fx=1;
	fy=1;
	cx=1;
	cy=1;
	camera_position = cvPoint3D64f(0,0,0);
	LED1_position = cvPoint3D64f(0,0,0);
	rotation = (double **)malloc(sizeof(double)*9);
	for(int i=0;i<3;i++)
	{
		rotation[i]=(double*)malloc(sizeof(double)*3);
		memset(rotation[i],0,sizeof(double)*3);
	}
	rotation_apriori = (double **)malloc(sizeof(double)*9);
	for(int i=0;i<3;i++)
	{
		rotation_apriori[i]=(double*)malloc(sizeof(double)*3);
		memset(rotation_apriori[i],0,sizeof(double)*3);
	}
	translation = (double *)malloc(sizeof(double)*3);
	memset(translation,0,sizeof(double)*3);
	left_eye_ROI = cvRect(0,0,1,1);
	right_eye_ROI = cvRect(0,0,1,1);
	left_eye_pupil = cvPoint2D64f(0,0);
	right_eye_pupil = cvPoint2D64f(0,0);
	Lucam=NULL;
}

GTCamera::~GTCamera(void)
{
	cvReleaseImage(&DarkFrame);
	DarkFrame = NULL;
	cvReleaseImage(&BrightFrame);
	BrightFrame = NULL;
	cvReleaseImage(&work);
	work = NULL;

	if (rotation!=NULL)
	{
		for(int i=0;i<3;i++)
		{
			if (rotation[i]!= NULL)free(rotation[i]);
		}
		free(rotation);
		rotation = NULL;
	}

	if (rotation_apriori!=NULL)
	{
		for(int i=0;i<3;i++)
		{
			if (rotation_apriori[i]!= NULL)free(rotation_apriori[i]);
		}
		free(rotation_apriori);
		rotation_apriori = NULL;
	}
	

	if (translation!=NULL) free(translation);
	translation = NULL;
}
GTCamera::GTCamera(const GTCamera &_cam)
{
	BrightFrame = _cam.BrightFrame;
	cx = _cam.cx;
	cy = _cam.cy;

	DarkFrame = _cam.DarkFrame;

	fx = _cam.fx;
	fy = _cam.fy;

	Lucam = _cam.Lucam;
	camera_position = cvPoint3D64f(_cam.camera_position.x,_cam.camera_position.y,_cam.camera_position.z);
	LED1_position = cvPoint3D64f(_cam.LED1_position.x,_cam.LED1_position.y,_cam.LED1_position.z);
	darkframeglintdetection=_cam.darkframeglintdetection;
	rotation = (double **)malloc(sizeof(double)*9);
	for(int i=0;i<3;i++)
	{
		rotation[i]=(double*)malloc(sizeof(double)*3);
		memcpy(rotation[i],_cam.rotation[i],sizeof(double)*3);
	}
	rotation_apriori = (double **)malloc(sizeof(double)*9);
	for(int i=0;i<3;i++)
	{
		rotation_apriori[i]=(double*)malloc(sizeof(double)*3);
		memcpy(rotation_apriori[i],_cam.rotation_apriori[i],sizeof(double)*3);
	}
	translation = (double *)malloc(sizeof(double)*3);
	memcpy(translation,_cam.translation,sizeof(double)*3);

	work = _cam.work;
	left_eye_planes = _cam.left_eye_planes;
	right_eye_planes = _cam.right_eye_planes;

	memcpy(&left_eye_ROI,&_cam.left_eye_ROI,sizeof(CvRect));
	memcpy(&right_eye_ROI,&_cam.right_eye_ROI,sizeof(CvRect));

	memcpy(&left_eye_pupil,&_cam.left_eye_pupil,sizeof(CvPoint2D64f));
	memcpy(&right_eye_pupil,&_cam.right_eye_pupil,sizeof(CvPoint2D64f));


}
void GTCamera::load_calib_data(char *_filename)
{
	double v[3] = {0, 0, 0};
	CvMat *oldrotation = cvCreateMat(3,3,CV_64FC1);

	FILE *fp = fopen(_filename,"r");
	fscanf(fp,"%lf", &fx);//fx
	fscanf(fp,"%lf", &cx);//cx
	fscanf(fp,"%lf", &fy);//fy
	fscanf(fp,"%lf", &cy);//cy

	//load rotation matrix and invert.
	fscanf(fp,"%lf", &v[0]);
	fscanf(fp,"%lf", &v[1]);
	fscanf(fp,"%lf", &v[2]);

	CvMat rotv = cvMat(1,3,CV_64FC1,v);
	cvRodrigues2(&rotv,oldrotation);
	double **rotation_temp = (double **)malloc(sizeof(double)*9);
	for(int i=0;i<3;i++)
	{
		rotation_temp[i]=(double*)malloc(sizeof(double)*3);
		for(int j=0; j<3;j++)
		{
			rotation_temp[i][j]=cvGetReal2D(oldrotation,i,j);
		}
	}	
	///memcpy(rotation,rotation_temp,sizeof(double)*9);
	invert_matrix(rotation_temp,rotation);
	//load tanslation and negate
	fscanf(fp,"%lf", &translation[0]);
	fscanf(fp,"%lf", &translation[1]);
	fscanf(fp,"%lf", &translation[2]);
	translation[0]*=-1.0;
	translation[1]*=-1.0;
	translation[2]*=-1.0;
	fclose(fp);



	

	

for(int i=0;i<3;i++)
	{
		free(rotation_temp[i]);
	}
	free(rotation_temp);
	cvReleaseMat(&oldrotation);

}
//loads the 3D position of the camera and the attached LED w.r.t the global coordinate system.
//format is space-separated float values(%6.4f) x y z (of cam = LED 0) x y z (of LED1)
void GTCamera::load_position_data(char *_filename)
{
	double v[3] = {0, 0, 0};
	CvMat *oldrotation = cvCreateMat(3,3,CV_64FC1);

	FILE *fp = fopen(_filename,"r");
	//read and load cam position
	fscanf(fp,"%lf", &v[0]);
	fscanf(fp,"%lf", &v[1]);
	fscanf(fp,"%lf", &v[2]);

	camera_position.x = v[0];
	camera_position.y = v[1];
	camera_position.z = v[2];

	//read and load LED1 position
	fscanf(fp,"%lf", &v[0]);
	fscanf(fp,"%lf", &v[1]);
	fscanf(fp,"%lf", &v[2]);

	LED1_position.x = v[0];
	LED1_position.y = v[1];
	LED1_position.z = v[2];

	fclose(fp);
}
//calculate f_x and f_y for a given horizontal opening angle of the camera.
//assumes 1280x1024 pictures
void GTCamera::calculate_focal_dist(double _angle)
{
	//convert to radian
	_angle*= 3.14159265358979323846/180.0;
	fx= 1280.0 * 0.5 / tan(_angle / 2.0);
	fy= 1024.0 *4.0/3.0* 0.5 / tan(_angle / 2.0);
}