// GazeTop.cpp : Defines the entry point for the console application.


#include "stdafx.h"
#include <highgui.h>
#include <cvcam.h>
#include <time.h>
#include <vector>
#include <list>
#include <string>
#include <fstream>
#include <iostream>
#include <sstream>

#include "GTCamera.h"
#include "lucamapi.h"
#include "image_processing.h"
#include "algebra.h"
#include "opengl.h"
#include "Table.h"
//Syntax: 
// FOO_: global variable
// FOO: local variable
// _FOO: parameter


DWORD WINAPI processcam0(LPVOID _data);
DWORD WINAPI processcam1(LPVOID _data);
DWORD WINAPI processcam2(LPVOID _data);
void process_image(IplImage *_image,const int _camnumber);
void stereo_callback(IplImage *_image0,IplImage *_image1);


typedef struct __blob_pair{
	int a,b;
} blob_pair;


//globals
std::vector<GTCamera> cams_;
CvFont font_;
Table table_;
sync_struct *sync;
int piccount=0;
CvSize picsize = cvSize(1280,1024);
int smooth_=0;
	

void DrawCross(IplImage *_image,CvPoint2D64f &_point)
{	
	cvLine(_image,cvPoint(_point.x-10,_point.y),cvPoint(_point.x+10,_point.y),CV_RGB(255,255,255));
	cvLine(_image,cvPoint(_point.x,_point.y-10),cvPoint(_point.x,_point.y+10),CV_RGB(255,255,255));

}

void parse_pov_file(const char *_filename)
{
	std::ifstream f;  
	std::string s;
	f.open(_filename, std::ios::in); 
	int camnumber=0;
	char buffer[255];
	bool skip_cam=false;
	bool skip_light=false;
	bool skip_rot=false;
	bool skip_angle=false;
	bool invert_rotation=false;//true iff direction=<0,0,-1> .. the rotationmatrix has a 180 degree turn about y-axis before the other rotations.
	while (!f.eof())          
	{
		itoa(camnumber,buffer,10);
		std::getline(f, s);        
		//find the cam pos,light_source,rotation angle,opening angle, and the point the user is focussing on(in case of povray generated images(
		std::string sought_cam="cam";
		sought_cam+= buffer;
		sought_cam+="=<";

		std::string sought_light="led";
		sought_light+=buffer;
		sought_light+="=<";

		std::string sought_rotate="rot";
		sought_rotate+=buffer;
		sought_rotate+="=<";

		std::string sought_angle="angle";
		sought_angle+=buffer;


		std::string sought_focus="focus=<";

		std::string sought_direction="direction_=<";

		std::stringstream tmp;
		int pos_cam = s.find(sought_cam);
		int pos_light = s.find(sought_light);
		int pos_rot = s.find(sought_rotate);
		int pos_angle = s.find(sought_angle);
		int pos_focus = s.find(sought_focus);
		int pos_direction = s.find(sought_direction);

		int start,end;
		if (pos_cam!=-1)
		{
			start=pos_cam+6;
			end=s.find(",",start);
			tmp << s.substr(start,end-start);
			tmp >> (cams_[camnumber].camera_position).x;
			tmp.clear();
			start=end+1;

			end=s.find(",",start);
			tmp << s.substr(start,end-start);
			tmp >> (cams_[camnumber].camera_position).y;
			tmp.clear();
			start=end+1;

			end=s.find(">",start);
			tmp << s.substr(start,end-start);
			tmp >> (cams_[camnumber].camera_position).z;
			tmp.clear();
		}
		if (pos_rot!=-1)
		{

			double x_angle,y_angle,z_angle;
			start=pos_rot+6;
			end=s.find(",",start);
			tmp << s.substr(start,end-start);
			tmp >> x_angle;
			tmp.clear();
			start=end+1;

			end=s.find(",",start);
			tmp << s.substr(start,end-start);
			tmp >> y_angle;
			tmp.clear();
			start=end+1;

			end=s.find(">",start);
			tmp << s.substr(start,end-start);
			tmp >> z_angle;
			tmp.clear();

			RotationMatrix(x_angle,y_angle,z_angle,invert_rotation,cams_[camnumber].rotation_apriori);

		}
#if POVRAYIMG

		if (pos_angle!=-1)
		{

			double opening_angle;
			start=pos_angle+7;
			tmp << s.substr(start,s.size());
			tmp >> opening_angle;
			tmp.clear();
			cams_[camnumber].calculate_focal_dist(opening_angle);

		}
#endif
		if (pos_light!=-1)
		{

			start=pos_light+6;
			end=s.find(",",start);
			tmp << s.substr(start,end-start);
			tmp >> (cams_[camnumber].LED1_position).x;
			tmp.clear();
			start=end+1;

			end=s.find(",",start);
			tmp << s.substr(start,end-start);
			tmp >> (cams_[camnumber].LED1_position).y;
			tmp.clear();
			start=end+1;

			end=s.find(">",start);
			tmp << s.substr(start,end-start);
			tmp >> (cams_[camnumber].LED1_position).z;
			tmp.clear();
			camnumber++;

		}
#if POVRAYIMG

		if (pos_focus!=-1)
		{
			//z is assumed to be the same as the display
			start=pos_focus+7;
			end=s.find(",",start);
			tmp << s.substr(start,end-start);
			tmp >> GTCamera::reference_gaze_focus.x;
			tmp.clear();
			start=end+1;

			end=s.find(",",start);
			tmp << s.substr(start,end-start);
			tmp >> GTCamera::reference_gaze_focus.y;
			tmp.clear();
			start=end+1;

			end=s.find(">",start);
			tmp << s.substr(start,end-start);
			tmp >> GTCamera::reference_gaze_focus.z;
			tmp.clear();

		}
#endif
		if (pos_direction!=-1)
		{
			//z is assumed to be the same as the display
			double z;
			start=pos_direction+12;
			end=s.find(",",start);
			start=end+1;

			end=s.find(",",start);
			start=end+1;

			end=s.find(">",start);
			tmp << s.substr(start,end-start);
			tmp >> z;
			tmp.clear();
			if (z==-1) invert_rotation=true;

		}

	}
	f.close();                // close file



}
std::string str_replace(const std::string& search, const std::string& replace, std::string subject)
{
	int pos = subject.find(search, 0);
	while(pos != std::string::npos)
	{
		subject.erase(pos, search.length());
		subject.insert(pos, replace);
		pos = subject.find(search, pos + replace.length());
	}
	return subject;
}
int _tmain(int argc, _TCHAR* argv[])
{
	int ncams = LucamNumCameras();//we assume at least 3 cameras

	cvInitFont(&font_,CV_FONT_VECTOR0,0.8,0.8,0.0,2.0);
	cams_.resize(3);//inits the cam structure 
	//cam0 is 60148381
	//cam1 is 60148406
	//cam2 is 60148346
	LUCAM_VERSION *serialinfo = (LUCAM_VERSION*) malloc (sizeof(LUCAM_VERSION)*ncams);
	LucamEnumCameras(serialinfo,ncams);

	//map to the right camera
	for(int i=0;i<ncams;i++)
	{
		if(serialinfo[i].serialnumber==60148381)
		{
			cams_[0].Lucam = LucamCameraOpen(i+1);
		}
		if(serialinfo[i].serialnumber==60148406)
		{
			cams_[1].Lucam = LucamCameraOpen(i+1);
		}
		if(serialinfo[i].serialnumber==60148346)
		{
			cams_[2].Lucam = LucamCameraOpen(i+1);
		}

	}	

	cvNamedWindow("0: webcam");
	cvNamedWindow("1: webcam");
	cvNamedWindow("2: webcam");
	cvNamedWindow("TableView");


	table_.load_position_data("table_position.cfg");

	parse_pov_file("GTmovie.pov");//

	//enter intrinsic values in case of no POVRAYIMG
#if !POVRAYIMG
	//the picturecentershift is subtracted from cx/cy 
	cams_[0].cx=640+27.97 ;
	cams_[0].cy=512-90.5;
	cams_[0].fx=4886.2666666666666666666666666667;
	cams_[0].fy=4886.2666666666666666666666666667;


	cams_[1].cx=640-0.83;
	cams_[1].cy=512-97.5;
	cams_[1].fx=4877.47;
	cams_[1].fy=4877.47;


	cams_[2].cx=640-8.93;
	cams_[2].cy=512-97.98;
	cams_[2].fx=4909.6;
	cams_[2].fy=4909.6;
#else
	cams_[0].cx=640;
	cams_[0].cy=512;
	cams_[1].cx=640;
	cams_[1].cy=512;
	cams_[2].cx=640;
	cams_[2].cy=512;
#endif

#ifdef _DEBUG

	cvNamedWindow("0: diff");
	cvNamedWindow("0: thresh_pupil");
	cvNamedWindow("0: ROI_right");
	cvNamedWindow("0: ROI_left");

	cvNamedWindow("1: diff");
	cvNamedWindow("1: thresh_pupil");
	cvNamedWindow("1: ROI_right");
	cvNamedWindow("1: ROI_left");

	cvNamedWindow("2: diff");
	cvNamedWindow("2: thresh_pupil");
	cvNamedWindow("2: ROI_right");
	cvNamedWindow("2: ROI_left");

#endif
	//start the OpenGLthread
	HANDLE hGlThread; 
	DWORD dwGlThreadID;
	sync = (sync_struct*)malloc(sizeof(sync_struct));
	sync->cams = &cams_;
	sync->killthread = false;

	hGlThread = CreateThread(NULL,0,GlMain,sync,0,&dwGlThreadID);

	//Setup the cameras
	LUCAM_SNAPSHOT lsSettings;
	memset(&lsSettings,0,sizeof(LUCAM_SNAPSHOT));

	lsSettings.exposure = 50.0f;
	lsSettings.gain = 1.0;
	lsSettings.gainRed = 1.0;
	lsSettings.gainBlue = 1.0;
	lsSettings.gainGrn1 = 1.0;
	lsSettings.gainGrn2 = 1.0;
	lsSettings.useStrobe = FALSE;
	lsSettings.strobeDelay = 0;
	FLOAT FrameRate = 15.0f;
	if (ncams>0) LucamGetFormat(cams_[0].Lucam, &lsSettings.format, &FrameRate);

	lsSettings.format.pixelFormat = LUCAM_PF_8;
	lsSettings.useHwTrigger = FALSE;
	lsSettings.timeout = 10000;
	lsSettings.shutterType = LUCAM_SHUTTER_TYPE_GLOBAL;;
	//lsSettings.shutterType = LUCAM_SHUTTER_TYPE_ROLLING;
	lsSettings.exposureDelay = 0;
	lsSettings.flReserved1 = 0.0;
	lsSettings.flReserved2 = 0.0;
	lsSettings.ulReserved1 = 0;
	lsSettings.ulReserved2 = 0;
	UCHAR *pcFrame = (UCHAR *) malloc (lsSettings.format.width * lsSettings.format.height * sizeof(UCHAR));

	for(int i=0;i<ncams;i++)
	{
		LucamEnableFastFrames(cams_[i].Lucam,&lsSettings);
		LucamGpoSelect(cams_[i].Lucam, 0xF);
	}

	IplImage *current_frame_cam0= cvCreateImage(cvSize(1280,1024),IPL_DEPTH_8U,1);
	IplImage *current_frame_cam1= cvCreateImage(cvSize(1280,1024),IPL_DEPTH_8U,1);
	IplImage *current_frame_cam2= cvCreateImage(cvSize(1280,1024),IPL_DEPTH_8U,1);

	std::vector<plane> right_eye_planes;
	right_eye_planes.resize(9);
	std::vector<plane> left_eye_planes;
	left_eye_planes.resize(9);

	bool cam0rightvalid = false;
	bool cam1rightvalid = false;
	bool cam2rightvalid = false;
	bool cam0leftvalid = false;
	bool cam1leftvalid = false;
	bool cam2leftvalid = false;

#ifndef _DEBUG	
	HANDLE hThread[3]; 
	DWORD dwThreadID[3]; 
#endif
	//eventloop
	//lighting scheme is:
	//Frame0:  all LEDS on. all CAMS off
	//Frame1:  all LEDS off. all CAMS on

	int frame=0;


#define NUM_VARIATIONS 100 //the number of variations of every parameter
	srand ( time(NULL) );
#if STATICIMG
	double randdouble=0;
	CvPoint3D64f randvector = cvPoint3D64f(0,0,0);
	FILE *fp = NULL;
#endif
	bool done=false;
	while(!done)
	{	
#if STATICIMG

		
		//save the unmodified gaze focus
		if(frame==3)
		{
			GTCamera::non_modified_gaze_focus.x=GTCamera::gaze_focus.x;
			GTCamera::non_modified_gaze_focus.y=GTCamera::gaze_focus.y;
			GTCamera::non_modified_gaze_focus.z=GTCamera::gaze_focus.z;
		}
		if(false&&(frame>3)&&(frame%2==0))//every second frame(i.e. every dark frame) we modify a parameter
		{
			//modify a parameter 
			switch((frame-4)/(2*NUM_VARIATIONS))
			{
			case 0: 
				//generate randomvector
				randvector.x = random_gauss(0,1);
				randvector.y = random_gauss(0,1);
				randvector.z = random_gauss(0,1);

				//vary the position of camera 0
				cams_[0].camera_position.x += randvector.x;
				cams_[0].camera_position.y += randvector.y;
				cams_[0].camera_position.z += randvector.z;
				break;

			case 1: 
				//generate randomvector
				randvector.x = random_gauss(0,1);
				randvector.y = random_gauss(0,1);
				randvector.z = random_gauss(0,1);

				//vary the position of camera 1
				cams_[1].camera_position.x += randvector.x;
				cams_[1].camera_position.y += randvector.y;
				cams_[1].camera_position.z += randvector.z;
				break;

			case 2: 
				//generate randomvector
				randvector.x = random_gauss(0,1);
				randvector.y = random_gauss(0,1);
				randvector.z = random_gauss(0,1);

				//vary the position of camera 2
				cams_[2].camera_position.x += randvector.x;
				cams_[2].camera_position.y += randvector.y;
				cams_[2].camera_position.z += randvector.z;
				break;
			case 3: 
				//generate randomvector
				randvector.x = random_gauss(0,10);
				randvector.y = random_gauss(0,10);
				randvector.z = random_gauss(0,10);

				//vary the position of LED0
				cams_[0].LED1_position.x += randvector.x;
				cams_[0].LED1_position.y += randvector.y;
				cams_[0].LED1_position.z += randvector.z;
				break;
			case 4: 
				//generate randomvector
				randvector.x = random_gauss(0,10);
				randvector.y = random_gauss(0,10);
				randvector.z = random_gauss(0,10);

				//vary the position of LED1
				cams_[1].LED1_position.x += randvector.x;
				cams_[1].LED1_position.y += randvector.y;
				cams_[1].LED1_position.z += randvector.z;
				break;
			case 5: 
				//generate randomvector
				randvector.x = random_gauss(0,10);
				randvector.y = random_gauss(0,10);
				randvector.z = random_gauss(0,10);

				//vary the position of LED2
				cams_[2].LED1_position.x += randvector.x;
				cams_[2].LED1_position.y += randvector.y;
				cams_[2].LED1_position.z += randvector.z;
				break;
			case 6: 
				{
					//generate random angle
					randdouble = random_gauss(0,0.5);

					//calculate rotation matrix
					double **rotx = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotx[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(randdouble,0,0,false,rotx);
					matrix_mul(cams_[0].rotation,rotx,cams_[0].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotx[i]);
					}
					free(rotx);

				}
				break;
			case 7: 
				{
					//generate random angle
					randdouble = random_gauss(0,0.5);

					//calculate rotation matrix
					double **roty = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						roty[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,randdouble,0,false,roty);
					matrix_mul(cams_[0].rotation,roty,cams_[0].rotation);
					for(int i=0;i<3;i++)
					{
						free(roty[i]);
					}
					free(roty);

				}
				break;
			case 8: 
				{
					//generate random angle
					randdouble = random_gauss(0,0.5);

					//calculate rotation matrix
					double **rotz = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotz[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,0,randdouble,false,rotz);
					matrix_mul(cams_[0].rotation,rotz,cams_[0].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotz[i]);
					}
					free(rotz);

				}
				break;
			case 9: 
				{
					//generate random angle
					randdouble = random_gauss(0,0.5);

					//calculate rotation matrix
					double **rotx = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotx[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(randdouble,0,0,false,rotx);
					matrix_mul(cams_[1].rotation,rotx,cams_[1].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotx[i]);
					}
					free(rotx);

				}
				break;
			case 10: 
				{
					//generate random angle
					randdouble = random_gauss(0,0.5);

					//calculate rotation matrix
					double **roty = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						roty[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,randdouble,0,false,roty);
					matrix_mul(cams_[1].rotation,roty,cams_[1].rotation);
					for(int i=0;i<3;i++)
					{
						free(roty[i]);
					}
					free(roty);

				}
				break;
			case 11: 
				{
					//generate random angle
					randdouble = random_gauss(0,0.5);

					//calculate rotation matrix
					double **rotz = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotz[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,0,randdouble,false,rotz);
					matrix_mul(cams_[1].rotation,rotz,cams_[1].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotz[i]);
					}
					free(rotz);

				}
				break;
			case 12: 
				{
					//generate random angle
					randdouble = random_gauss(0,0.5);

					//calculate rotation matrix
					double **rotx = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotx[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(randdouble,0,0,false,rotx);
					matrix_mul(cams_[2].rotation,rotx,cams_[2].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotx[i]);
					}
					free(rotx);

				}
				break;
			case 13: 
				{
					//generate random angle
					randdouble = random_gauss(0,0.5);

					//calculate rotation matrix
					double **roty = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						roty[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,randdouble,0,false,roty);
					matrix_mul(cams_[2].rotation,roty,cams_[2].rotation);
					for(int i=0;i<3;i++)
					{
						free(roty[i]);
					}
					free(roty);

				}
				break;
			case 14: 
				{
					//generate random angle
					randdouble = random_gauss(0,0.5);

					//calculate rotation matrix
					double **rotz = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotz[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,0,randdouble,false,rotz);
					matrix_mul(cams_[2].rotation,rotz,cams_[2].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotz[i]);
					}
					free(rotz);

				}
				break;
			case 15:
				{
					//generate random scale factor
					randdouble=random_gauss(1,0.01);
					set_between(randdouble,0.01,100.0);//make sure its not zero

					cams_[0].cx*=randdouble;
					cams_[0].cy*=randdouble;
				}
				break;
			case 16:
				{
					//generate random scale factor
					randdouble=random_gauss(1,0.01);
					set_between(randdouble,0.01,100.0);//make sure its not zero

					cams_[1].cx*=randdouble;
					cams_[1].cy*=randdouble;
				}
				break;
				case 17:
				{
					//generate random scale factor
					randdouble=random_gauss(1,0.01);
					set_between(randdouble,0.01,100.0);//make sure its not zero

					cams_[2].cx*=randdouble;
					cams_[2].cy*=randdouble;
				}
				break;
				case 18:
				{
					//generate random scale factor
					randdouble=random_gauss(1,0.01);
					set_between(randdouble,0.01,100.0);//make sure its not zero

					cams_[0].fx*=randdouble;
					cams_[0].fy*=randdouble;
				}
				break;
				case 19:
				{
					//generate random scale factor
					randdouble=random_gauss(1,0.01);
					set_between(randdouble,0.01,100.0);//make sure its not zero

					cams_[1].fx*=randdouble;
					cams_[1].fy*=randdouble;
				}
				break;
				case 20:
				{
					//generate random scale factor
					randdouble=random_gauss(1,0.01);
					set_between(randdouble,0.01,100.0);//make sure its not zero

					cams_[2].fx*=randdouble;
					cams_[2].fy*=randdouble;
				}
				break;
			case 21:
				{
					
					//generate random smoothing factor. .must be >=1 and odd
					smooth_=random_gauss(1,3);
					set_between(smooth_,0,100);
					smooth_=smooth_*2+1;


				}
				break;
			}

		}


#endif

		//shift to origin and scale by width and height of the table
#if POVRAYIMG									
		table_.reference_impactpoint.x = (GTCamera::reference_gaze_focus.x-table_.p1.x)/(table_.p2.x-table_.p1.x);
		table_.reference_impactpoint.y = (table_.p1.y-GTCamera::reference_gaze_focus.y)/(table_.p1.y-table_.p3.y);
#endif
		//TODO insert referenc_impactpoint for non POVRAYIMG
		//set up the new lighting scheme

		switch(frame % 2)
		{
		case 0: 
			if(cams_[0].Lucam!=NULL) LucamGpioWrite(cams_[0].Lucam,(unsigned char) 4);
			cams_[0].darkframeglintdetection = true;
			if(cams_[1].Lucam!=NULL) LucamGpioWrite(cams_[1].Lucam,(unsigned char) 4);
			cams_[1].darkframeglintdetection = true;
			if(cams_[2].Lucam!=NULL) LucamGpioWrite(cams_[2].Lucam,(unsigned char) 4);
			cams_[2].darkframeglintdetection = true;
			break;
		case 1:
			if(cams_[0].Lucam!=NULL) LucamGpioWrite(cams_[0].Lucam,(unsigned char) 2);
			cams_[0].darkframeglintdetection = false;
			if(cams_[1].Lucam!=NULL) LucamGpioWrite(cams_[1].Lucam,(unsigned char) 2);
			cams_[1].darkframeglintdetection = false;
			if(cams_[2].Lucam!=NULL) LucamGpioWrite(cams_[2].Lucam,(unsigned char) 2);
			cams_[2].darkframeglintdetection = false;
			break;
		}
#if STATICIMG == 0

		LucamTakeFastFrame(cams_[0].Lucam,pcFrame);
		//convert to IplImage	
		memcpy((current_frame_cam0->imageData),pcFrame ,lsSettings.format.width * lsSettings.format.height * sizeof(UCHAR));
		LucamTakeFastFrame(cams_[1].Lucam,pcFrame);
		//convert to IplImage
		memcpy((current_frame_cam1->imageData),pcFrame ,lsSettings.format.width * lsSettings.format.height * sizeof(UCHAR));
		LucamTakeFastFrame(cams_[2].Lucam,pcFrame);
		//convert to IplImage
		memcpy((current_frame_cam2->imageData),pcFrame ,lsSettings.format.width * lsSettings.format.height * sizeof(UCHAR));
#endif

		/*	if ((cvWaitKey(1) & 255)==32)
		{
		char buffer[256];
		std::string write_prefix;

		sprintf_s(buffer,"cam1.%4d.jpg",frame);
		write_prefix = std::string(buffer);		
		cvSaveImage(str_replace(" ","0",write_prefix).c_str(),current_frame_cam0);
		sprintf_s(buffer,"cam2.%4d.jpg",frame);
		write_prefix = std::string(buffer);		
		cvSaveImage(str_replace(" ","0",write_prefix).c_str(),current_frame_cam1);
		sprintf_s(buffer,"cam3.%4d.jpg",frame);
		write_prefix = std::string(buffer);		
		cvSaveImage(str_replace(" ","0",write_prefix).c_str(),current_frame_cam2);

		}*/
		//create thread and process the image
#ifndef _DEBUG	
		hThread[0] = CreateThread(NULL,0,processcam0,current_frame_cam0,0,&dwThreadID[0]);
		hThread[1] = CreateThread(NULL,0,processcam1,current_frame_cam1,0,&dwThreadID[1]);

		hThread[2] = CreateThread(NULL,0,processcam2,current_frame_cam2,0,&dwThreadID[2]);

		//wait for the worker threads to finish
		WaitForMultipleObjects(3,hThread,TRUE,INFINITE);

		//and close them
		CloseHandle(hThread[0]);
		CloseHandle(hThread[1]); 
		CloseHandle(hThread[2]); 
#endif 



		//highgui is not threadsafe. therefore we can't use threads while debugging
#ifdef _DEBUG
		processcam0(current_frame_cam0);
		processcam1(current_frame_cam1);
		processcam2(current_frame_cam2);
#endif

#ifndef _DEBUG
		DrawCross(current_frame_cam0,cams_[0].left_eye_pupil);
		DrawCross(current_frame_cam0,cams_[0].right_eye_pupil);
		DrawCross(current_frame_cam1,cams_[1].left_eye_pupil);
		DrawCross(current_frame_cam1,cams_[1].right_eye_pupil);
		DrawCross(current_frame_cam2,cams_[2].left_eye_pupil);
		DrawCross(current_frame_cam2,cams_[2].right_eye_pupil);
#endif
		cvShowImage("0: webcam",current_frame_cam0);
		cvShowImage("1: webcam",current_frame_cam1);
		cvShowImage("2: webcam",current_frame_cam2);

		//planes = cam0.planes U cam1.planes U cam2.planes

		for(int i=0;i< cams_[0].right_eye_planes.size();i++)
		{
			right_eye_planes[i].d = cams_[0].right_eye_planes[i].d;
			right_eye_planes[i].n_x = cams_[0].right_eye_planes[i].n_x;
			right_eye_planes[i].n_y = cams_[0].right_eye_planes[i].n_y;
			right_eye_planes[i].n_z = cams_[0].right_eye_planes[i].n_z;
			cam0rightvalid = true;

		}
		for(int i=0;i< cams_[1].right_eye_planes.size();i++)
		{
			right_eye_planes[i+cams_[0].right_eye_planes.size()].d = cams_[1].right_eye_planes[i].d;
			right_eye_planes[i+cams_[0].right_eye_planes.size()].n_x = cams_[1].right_eye_planes[i].n_x;
			right_eye_planes[i+cams_[0].right_eye_planes.size()].n_y = cams_[1].right_eye_planes[i].n_y;
			right_eye_planes[i+cams_[0].right_eye_planes.size()].n_z = cams_[1].right_eye_planes[i].n_z;	
			cam1rightvalid = true;
		}
		for(int i=0;i< cams_[2].right_eye_planes.size();i++)
		{
			right_eye_planes[i+cams_[1].right_eye_planes.size()+cams_[0].right_eye_planes.size()].d = cams_[2].right_eye_planes[i].d;
			right_eye_planes[i+cams_[1].right_eye_planes.size()+cams_[0].right_eye_planes.size()].n_x = cams_[2].right_eye_planes[i].n_x;
			right_eye_planes[i+cams_[1].right_eye_planes.size()+cams_[0].right_eye_planes.size()].n_y = cams_[2].right_eye_planes[i].n_y;
			right_eye_planes[i+cams_[1].right_eye_planes.size()+cams_[0].right_eye_planes.size()].n_z = cams_[2].right_eye_planes[i].n_z;	
			cam2rightvalid = true;
		}
		for(int i=0;i< cams_[0].left_eye_planes.size();i++)
		{
			left_eye_planes[i].d = cams_[0].left_eye_planes[i].d;
			left_eye_planes[i].n_x = cams_[0].left_eye_planes[i].n_x;
			left_eye_planes[i].n_y = cams_[0].left_eye_planes[i].n_y;
			left_eye_planes[i].n_z = cams_[0].left_eye_planes[i].n_z;
			cam0leftvalid = true;

		}
		for(int i=0;i< cams_[1].left_eye_planes.size();i++)
		{
			left_eye_planes[i+cams_[0].left_eye_planes.size()].d = cams_[1].left_eye_planes[i].d;
			left_eye_planes[i+cams_[0].left_eye_planes.size()].n_x = cams_[1].left_eye_planes[i].n_x;
			left_eye_planes[i+cams_[0].left_eye_planes.size()].n_y = cams_[1].left_eye_planes[i].n_y;
			left_eye_planes[i+cams_[0].left_eye_planes.size()].n_z = cams_[1].left_eye_planes[i].n_z;	
			cam1leftvalid = true;
		}
		for(int i=0;i< cams_[2].left_eye_planes.size();i++)
		{
			left_eye_planes[i+cams_[1].left_eye_planes.size()+cams_[0].left_eye_planes.size()].d = cams_[2].left_eye_planes[i].d;
			left_eye_planes[i+cams_[1].left_eye_planes.size()+cams_[0].left_eye_planes.size()].n_x = cams_[2].left_eye_planes[i].n_x;
			left_eye_planes[i+cams_[1].left_eye_planes.size()+cams_[0].left_eye_planes.size()].n_y = cams_[2].left_eye_planes[i].n_y;
			left_eye_planes[i+cams_[1].left_eye_planes.size()+cams_[0].left_eye_planes.size()].n_z = cams_[2].left_eye_planes[i].n_z;	
			cam2leftvalid = true;
		}
		//	calc eye position (planes) as soon as we have three sets of planes from the cameras
		if( cam0rightvalid && cam1rightvalid && cam2rightvalid)
		{

			//memset(&right_eye_planes[6],0,3*sizeof(plane));//disable values from camera 2
			point_nearest_to_planes(&right_eye_planes[0],9,GTCamera::right_eye_center);

#ifdef _DEBUG
			char str[255];
			sprintf(str,"Right Eye pos: %4.2lf,%4.2lf,%4.2lf",GTCamera::right_eye_center.x,GTCamera::right_eye_center.y,GTCamera::right_eye_center.z);
			cvPutText(current_frame_cam0,str,cvPoint(200,200),&font_,CV_RGB(255,255,255));

#endif
		}
		//	calc eye position(planes) as soon as we have two sets of planes from the cameras
		if( cam0leftvalid && cam1leftvalid && cam2leftvalid)
		{
			//memset(&left_eye_planes[6],0,3*sizeof(plane));//disable values from camera 2
			point_nearest_to_planes(&left_eye_planes[0],9,GTCamera::left_eye_center);
			char str[255];
#ifdef _DEBUG
			sprintf(str,"Left Eye pos: %4.2lf,%4.2lf,%4.2lf",GTCamera::left_eye_center.x,GTCamera::left_eye_center.y,GTCamera::left_eye_center.z);
			cvPutText(current_frame_cam0,str,cvPoint(200,250),&font_,CV_RGB(255,255,255));
			//cvShowImage("web",current_frame_cam0);
#endif
		}

		//update rotation matrices to fit the eye position
		adaptive_calibration(cams_,left_eye_planes);

		// calculate the gaze;
		calc_gaze(cams_,table_); 


		//visualize the gaze points TODO fancy
		IplImage *Table = cvCreateImage(cvSize(1280,768),IPL_DEPTH_8U,3);
		char str[255];
		CvPoint gazecenter = cvPoint(table_.impactpoint.x*Table->width,(table_.impactpoint.y)*Table->height);

		//linear mapping using calibration data
		int new_x,new_y;
		new_x = 19.71*gazecenter.x -11640;
		new_y = -5.56*gazecenter.y +4439;
		gazecenter.x=new_x;
		gazecenter.y=new_y;
		sprintf(str,"Gaze Focus: %4.2lf,%4.2lf,%4.2lf",GTCamera::gaze_focus.x,GTCamera::gaze_focus.y,GTCamera::gaze_focus.z);
		cvPutText(current_frame_cam0,str,cvPoint(200,300),&font_,CV_RGB(255,255,255));

		cvCircle(Table,gazecenter,20,CV_RGB(255,0,0),-1);
		cvCircle(Table,cvPoint(table_.reference_impactpoint.x*Table->width,table_.reference_impactpoint.y*Table->height),20,CV_RGB(0,0,255),-1);
		cvCircle(Table,cvPoint(50,50),20,CV_RGB(0,255,0),-1);
		cvCircle(Table,cvPoint(640,50),20,CV_RGB(0,255,0),-1);
		cvCircle(Table,cvPoint(1200,50),20,CV_RGB(0,255,0),-1);
		cvCircle(Table,cvPoint(50,384),20,CV_RGB(0,255,0),-1);
		cvCircle(Table,cvPoint(640,384),20,CV_RGB(0,255,0),-1);
		cvCircle(Table,cvPoint(1200,384),20,CV_RGB(0,255,0),-1);
		cvCircle(Table,cvPoint(50,700),20,CV_RGB(0,255,0),-1);
		cvCircle(Table,cvPoint(640,700),20,CV_RGB(0,255,0),-1);
		cvCircle(Table,cvPoint(1200,700),20,CV_RGB(0,255,0),-1);

		//cvCircle(Table,cvPoint(gazecenter),20,CV_RGB(0,255,0),-1);

		cvShowImage("TableView",Table);
		cvReleaseImage(&Table);
		Table=NULL;

		double difference_angle = calculateQuality(cams_,table_);


		//undo the adaptive calibration
		for(int j=0;j<3;j++)
		{
			for(int k=0;k<3;k++)
			{
				memcpy(cams_[j].rotation[k],cams_[j].rotation_apriori[k],3*sizeof(double));
			}

		}
		if( (cvWaitKey(1) & 255) == 27 ) done=true;
#if STATICIMG
		if ((frame>3)&&(frame%2==0))//revert the changes and save the quality
		{

			switch((frame-4)/(2*NUM_VARIATIONS))
			{
			case 0: 
				fp = fopen("cam0_translate.txt","a");
				//revert the position of camera 0
				cams_[0].camera_position.x -= randvector.x;
				cams_[0].camera_position.y -= randvector.y;
				cams_[0].camera_position.z -= randvector.z;

				if(frame!=0)
					fprintf(fp,"%f %f\n",difference_angle,sqrt(pow(randvector.x,2)+pow(randvector.y,2)+pow(randvector.z,2)));
				fclose(fp);
				break;

			case 1: 
				fp = fopen("cam1_translate.txt","a");
				//revert the position of camera 1
				cams_[1].camera_position.x -= randvector.x;
				cams_[1].camera_position.y -= randvector.y;
				cams_[1].camera_position.z -= randvector.z;

				if(frame!=0)
					fprintf(fp,"%f %f\n",difference_angle,sqrt(pow(randvector.x,2)+pow(randvector.y,2)+pow(randvector.z,2)));
				fclose(fp);	
				break;

			case 2: 
				fp = fopen("cam2_translate.txt","a");
				//revert the position of camera 2
				cams_[2].camera_position.x -= randvector.x;
				cams_[2].camera_position.y -= randvector.y;
				cams_[2].camera_position.z -= randvector.z;

				if(frame!=0)
					fprintf(fp,"%f %f\n",difference_angle,sqrt(pow(randvector.x,2)+pow(randvector.y,2)+pow(randvector.z,2)));
				fclose(fp);	
				break;

			case 3: 
				fp = fopen("LED0_translate.txt","a");
				//revert the position of LED0
				cams_[0].LED1_position.x -= randvector.x;
				cams_[0].LED1_position.y -= randvector.y;
				cams_[0].LED1_position.z -= randvector.z;

				if(frame!=0)
					fprintf(fp,"%f %f\n",difference_angle,sqrt(pow(randvector.x,2)+pow(randvector.y,2)+pow(randvector.z,2)));
				fclose(fp);	
				break;
			case 4: 
				fp = fopen("LED1_translate.txt","a");
				//revert the position of LED1
				cams_[1].LED1_position.x -= randvector.x;
				cams_[1].LED1_position.y -= randvector.y;
				cams_[1].LED1_position.z -= randvector.z;

				if(frame!=0)
					fprintf(fp,"%f %f\n",difference_angle,sqrt(pow(randvector.x,2)+pow(randvector.y,2)+pow(randvector.z,2)));
				fclose(fp);	
				break;
			case 5: 
				fp = fopen("LED2_translate.txt","a");
				//revert the position of LED2
				cams_[2].LED1_position.x -= randvector.x;
				cams_[2].LED1_position.y -= randvector.y;
				cams_[2].LED1_position.z -= randvector.z;

				if(frame!=0)
					fprintf(fp,"%f %f\n",difference_angle,sqrt(pow(randvector.x,2)+pow(randvector.y,2)+pow(randvector.z,2)));
				fclose(fp);	
				break;

			case 6:
				{
					fp = fopen("x_angle_rotate_cam0.txt","a");

					double **rotx = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotx[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(-randdouble,0,0,false,rotx);
					matrix_mul(cams_[0].rotation,rotx,cams_[0].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotx[i]);
					}
					free(rotx);
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
			case 7:
				{
					fp = fopen("y_angle_rotate_cam0.txt","a");

					double **roty = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						roty[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,-randdouble,0,false,roty);
					matrix_mul(cams_[0].rotation,roty,cams_[0].rotation);
					for(int i=0;i<3;i++)
					{
						free(roty[i]);
					}
					free(roty);
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
			case 8:
				{
					fp = fopen("z_angle_rotate_cam0.txt","a");

					double **rotz = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotz[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,0,-randdouble,false,rotz);
					matrix_mul(cams_[0].rotation,rotz,cams_[0].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotz[i]);
					}
					free(rotz);
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;

			case 9:
				{
					fp = fopen("x_angle_rotate_cam1.txt","a");

					double **rotx = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotx[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(-randdouble,0,0,false,rotx);
					matrix_mul(cams_[1].rotation,rotx,cams_[1].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotx[i]);
					}
					free(rotx);
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
			case 10:
				{
					fp = fopen("y_angle_rotate_cam1.txt","a");

					double **roty = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						roty[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,-randdouble,0,false,roty);
					matrix_mul(cams_[1].rotation,roty,cams_[1].rotation);
					for(int i=0;i<3;i++)
					{
						free(roty[i]);
					}
					free(roty);
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
			case 11:
				{
					fp = fopen("z_angle_rotate_cam1.txt","a");

					double **rotz = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotz[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,0,-randdouble,false,rotz);
					matrix_mul(cams_[1].rotation,rotz,cams_[1].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotz[i]);
					}
					free(rotz);
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;

			case 12:
				{
					fp = fopen("x_angle_rotate_cam2.txt","a");

					double **rotx = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotx[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(-randdouble,0,0,false,rotx);
					matrix_mul(cams_[2].rotation,rotx,cams_[2].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotx[i]);
					}
					free(rotx);
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
			case 13:
				{
					fp = fopen("y_angle_rotate_cam2.txt","a");

					double **roty = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						roty[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,-randdouble,0,false,roty);
					matrix_mul(cams_[2].rotation,roty,cams_[2].rotation);
					for(int i=0;i<3;i++)
					{
						free(roty[i]);
					}
					free(roty);
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
			case 14:
				{
					fp = fopen("z_angle_rotate_cam2.txt","a");

					double **rotz = (double **)malloc(sizeof(double)*3);
					for(int i=0;i<3;i++)
					{
						rotz[i]=(double*)malloc(sizeof(double)*3);
					}
					RotationMatrix(0,0,-randdouble,false,rotz);
					matrix_mul(cams_[2].rotation,rotz,cams_[2].rotation);
					for(int i=0;i<3;i++)
					{
						free(rotz[i]);
					}
					free(rotz);
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
			case 15:
				{
					fp = fopen("cx_cy_scale_cam0.txt","a");

					cams_[0].cx/=randdouble;
					cams_[0].cy/=randdouble;
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
				case 16:
				{
					fp = fopen("cx_cy_scale_cam1.txt","a");

					cams_[1].cx/=randdouble;
					cams_[1].cy/=randdouble;
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
				case 17:
				{
					fp = fopen("cx_cy_scale_cam2.txt","a");

					cams_[2].cx/=randdouble;
					cams_[2].cy/=randdouble;
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
				case 18:
				{
					fp = fopen("fx_fy_scale_cam0.txt","a");

					cams_[0].fx/=randdouble;
					cams_[0].fy/=randdouble;
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
				case 19:
				{
					fp = fopen("fx_fy_scale_cam1.txt","a");

					cams_[1].fx/=randdouble;
					cams_[1].fy/=randdouble;
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
			case 20:
				{
					fp = fopen("fx_fy_scale_cam2.txt","a");

					cams_[2].fx/=randdouble;
					cams_[2].fy/=randdouble;
					if(frame!=0)
						fprintf(fp,"%f %f\n",difference_angle,randdouble);
					fclose(fp);	
				}
				break;
			case 21:
				{
					
					fp = fopen("resolution_scale.txt","a");
					

					if(frame!=0)
						fprintf(fp,"%f %d\n",difference_angle,smooth_);
					fclose(fp);

				}
				break;

			
			default:
				done=true;
				break;
			}


		}
#endif 
		frame++;

	}

	cvReleaseImage(&current_frame_cam0);
	cvReleaseImage(&current_frame_cam1);
	cvReleaseImage(&current_frame_cam2);
	for(int i=0;i <ncams;i++)
	{
		LucamDisableFastFrames(cams_[i].Lucam);
		LucamCameraClose(cams_[i].Lucam);
	}

	free(pcFrame);
	pcFrame = NULL;

	//destroy GlWindow
	sync->killthread = true;
	WaitForMultipleObjects(1,&hGlThread,TRUE,INFINITE); 
	CloseHandle(hGlThread);
	free(sync);

	cvDestroyAllWindows();
	cams_.clear();

	return 2;

}

DWORD WINAPI processcam0(LPVOID _data)
{
	process_image((IplImage*)_data,0);
	return 0;
}

DWORD WINAPI processcam1(LPVOID _data)
{
	process_image((IplImage*)_data,1);
	return 0;
}

DWORD WINAPI processcam2(LPVOID _data)
{
	process_image((IplImage*)_data,2);
	return 0;
}
void process_image(IplImage *_image,const int _camnumber)
{

	GTCamera *cam = &(cams_[_camnumber]);

	piccount++;

	char buffer[255];	

	sprintf_s(buffer,"%d",_camnumber);
	std::string prefix = std::string(buffer);
	prefix+= ": ";
	sprintf_s(buffer,"%d_%d",piccount,_camnumber);
	std::string write_prefix = std::string(buffer);
#if POVRAYIMG
	sprintf_s(buffer,"GTmovie%d",2*_camnumber+1+(cam->darkframeglintdetection?0:1));
	std::string load_prefix = std::string(buffer);
#else
	sprintf_s(buffer,"%d",_camnumber);
	std::string load_prefix = std::string(buffer);
#endif

	if(!cam->work||(
		(cam->work->height!=picsize.height)||
		(cam->work->width!=picsize.width))
		)
	{
		cam->work = cvCreateImage(picsize,IPL_DEPTH_8U,1);//defines our working size
	}
	cvResize(_image,cam->work);


	CvSize size = cvGetSize(cam->work);

	if (!cam->DarkFrame||(
		(cam->DarkFrame->height!=picsize.height)||
		(cam->DarkFrame->width!=picsize.width)) 
		)
	{
		cam->DarkFrame = cvCreateImage( size, IPL_DEPTH_8U, 1);

	}
	if (!cam->BrightFrame||(
		(cam->BrightFrame->height!=picsize.height)||
		(cam->BrightFrame->width!=picsize.width))
		)
	{
		cam->BrightFrame= cvCreateImage( size, IPL_DEPTH_8U, 1);

	}

	
	IplImage *Frame;
#if STATICIMG 
	IplImage *load;
#endif
	bool detect_glint=true;//only detect the glint in a darkframe, because we need at least 3 glints for the eye pos.
	//point to the correct frames
	if(!cam->darkframeglintdetection)
	{
		Frame = cam->BrightFrame;
		detect_glint = false;


#if STATICIMG	
#if POVRAYIMG
		load = cvLoadImage((load_prefix +".BMP").c_str(),0);
#else
		load= cvLoadImage((load_prefix +"BRIGHT_TR.jpg").c_str(),0);
#endif
#else
		cvCopy(cam->work,Frame);
		cvSaveImage((write_prefix + "BRIGHT_MM.jpg").c_str(),Frame);
#endif
	}
	else
	{
		Frame = cam->DarkFrame;
		detect_glint = true;

#if STATICIMG		
#if POVRAYIMG

		load = cvLoadImage((load_prefix +".BMP").c_str(),0);
#else
		load= cvLoadImage((load_prefix +"DARK_TR.jpg").c_str(),0);
#endif
#else
		cvCopy(cam->work,Frame);
		cvSaveImage((write_prefix + "DARK_MM.jpg").c_str(),Frame);
#endif
	}

	
#if STATICIMG
	if(smooth_!=0)
		cvSmooth(load,load,CV_BLUR,smooth_);
	cvResize(load,Frame);	
	cvResize(Frame,_image);cvReleaseImage(&load);
#endif

	cvSaveImage((write_prefix + "input.jpg").c_str(),Frame);
	if ( (cam->DarkFrame)&&(cam->BrightFrame))//all frames are valid, compute the diff
	{
		//pupil calculation
		IplImage *diff=cvCreateImage(size,IPL_DEPTH_8U,1);
		IplImage *thresh=cvCreateImage(size,IPL_DEPTH_8U,1);
		//compensate for the different lighting. lower the brightness of the brightframe
		IplImage *tmp = cvCreateImage(cvSize(cam->DarkFrame->width,cam->DarkFrame->height),IPL_DEPTH_8U,1);
#if POVRAYIMG
		subtract(cam->BrightFrame,cam->DarkFrame,diff);
#else
		cvScale(cam->BrightFrame,tmp,0.8*cvMean(cam->DarkFrame)/cvMean(cam->BrightFrame));
		subtract(tmp,cam->DarkFrame,diff);
#endif
		cvReleaseImage(&tmp);
		tmp=NULL;


#ifdef _DEBUG
		cvShowImage((prefix + "diff").c_str(),diff);	
		//cvSaveImage((write_prefix + "diff.jpg").c_str(),diff);
#endif
		//remove noise
#if POVRAYIMG
		cvErode(diff,diff,NULL,2);
#else
		//cvErode(diff,diff,NULL,1);
#endif
		CvSize sz = cvSize(diff->width & - 2, diff->height & -2 );
		IplImage * pyr = cvCreateImage(cvSize(sz.width/2,sz.height/2), diff->depth, diff->nChannels);
		cvPyrDown(diff,pyr,7);
		cvPyrUp(pyr, diff, 7);
		cvReleaseImage(&pyr);


		//remove the semi bright parts
#if POVRAYIMG
		adaptiveThreshold(diff,thresh,0.97);
#else
		adaptiveThreshold(diff,thresh,0.95);
#endif
		//resize to a quarter of its size.. improves performance a lot
		IplImage *thresh_small=cvCreateImage(cvSize(size.width/2,size.height/2),IPL_DEPTH_8U,1);
		cvResize(thresh,thresh_small);
		CBlobResult blobs;
		blobs = CBlobResult(thresh_small,NULL,1,true);
		//blobs.PrintBlobs( "blobs.txt" );

		// discard some blobs
		blobs.Filter(blobs,B_INCLUDE,CBlobGetArea(),B_GREATER, 25);
		//		blobs.Filter(blobs,B_EXCLUDE,CBlobGetArea(),B_GREATER,1000);
		blobs.Filter(blobs,B_EXCLUDE,CBlobGetExterior(),B_GREATER,0);
		//blobs.Filter(blobs,B_EXCLUDE,CBlobGetCompactness(),B_GREATER,2.5);
		//calculate a list of blob pairs
		std::list<blob_pair> Bpairs;


		sprintf_s(buffer,"%d",_camnumber);
		std::string cam_prefix = std::string(buffer);

		//blobs.PrintBlobs((char*)(cam_prefix + "blobs.txt").c_str());

#if POVRAYIMG

		double mindist = 5;
#else
		double mindist = 60;
#endif

		double hdist, vdist;

		//accept every pair with distance greater than "mindist" pixel

		for(int acount =0; acount<blobs.GetNumBlobs();acount++)
		{
			//only consider one pair once.
			for(int bcount =acount+1;bcount<blobs.GetNumBlobs();bcount++)
			{
				CBlob *a = blobs.GetBlob(acount);
				CBlob *b = blobs.GetBlob(bcount);
				CBlob *left,*right,*top,*bottom;
				if (a->MinX() < b->MinX())
				{
					left = a;
					right = b;
				}
				else 
				{	
					left = b;
					right = a;
				}
				hdist = right->MinX() - left->MaxX();
				if(a->MinY() < b->MinY())
				{
					top = a;
					bottom =b;
				}
				else
				{
					top = b;
					bottom = a;
				}
				vdist = bottom->MinY() - top->MaxY();
				if(sqrt(pow(hdist,2) + pow(vdist,2)) > mindist)
				{
					//it doesnt matter, that blob a can be != to acount
					//because the pairs are the same i.e. dist(a,b) = dist(b,a)
					Bpairs.resize(Bpairs.size()+1);
					Bpairs.back().a=acount;
					Bpairs.back().b=bcount;
				}
			}

		}

		//discard all pairs who are not on the same height.. i.e. the line between the blob centers has an angle of more than 20 degrees
		std::list<blob_pair> Bpairs2;
		std::list<blob_pair>::iterator iter;


		for(iter = Bpairs.begin(); iter != Bpairs.end();iter++)
		{
			double cos_angle;
			double xvec,yvec;
			CBlob *left,*right;
			//sort by the x-coord
			if(blobs.GetBlob(iter->a)->MinX() <= blobs.GetBlob(iter->b)->MinX())
			{
				left = blobs.GetBlob(iter->a);
				right = blobs.GetBlob(iter->b);
			}
			else
			{
				left = blobs.GetBlob(iter->b);
				right = blobs.GetBlob(iter->a);
			}
			xvec = (right->MinX() + right->MaxX())/2.0 - (left->MinX() + left->MaxX())/2.0;
			yvec = (right->MinY() + right->MaxY())/2.0 - (left->MinY() + left->MaxY())/2.0;
			cos_angle = xvec/(sqrt(pow(xvec,2)+pow(yvec,2)));
#if !POVRAYIMG
			if (cos_angle >= 0.94)
#endif
				Bpairs2.push_back(*iter);
		}
		Bpairs.clear();

		std::list<blob_pair>::iterator brightest;


		brightest = Bpairs2.begin();
		double brightest_mean,current_mean;
		if(Bpairs2.size() >0) brightest_mean =( ((blobs.GetBlob(brightest->a))->Mean()*(blobs.GetBlob(brightest->a))->Area()) + 
			((blobs.GetBlob(brightest->b))->Mean()*(blobs.GetBlob(brightest->b))->Area()) )/ ( (blobs.GetBlob(brightest->a))->Area() + (blobs.GetBlob(brightest->b))->Area() );
		//select brightest blob pair. 
		//pairs brightness is the  brightness of each blob weighted by its size.
		for(iter = Bpairs2.begin(); iter != Bpairs2.end();iter++)
		{
			current_mean = ( ((blobs.GetBlob(iter->a))->Mean()*(blobs.GetBlob(iter->a))->Area()) + 
				((blobs.GetBlob(iter->b))->Mean()*(blobs.GetBlob(iter->b))->Area()) )/ ( (blobs.GetBlob(iter->a))->Area() + (blobs.GetBlob(iter->b))->Area() );
			if(brightest_mean < current_mean)
			{ 
				brightest = iter;
				brightest_mean = current_mean;
			}

		}

		// two step pupil center detection
		//1. roughly detect the eyes in the image
		//2. set the ROI to the eyes area and then find the pupil in it
		if(Bpairs2.size()>0)
		{
			CBlob left_iris,right_iris,tmp;
			int pupilwidth,pupilheight;


			//the 2 brightest blobs should be the left and the right iris.
			left_iris=blobs.GetBlob(brightest->a);
			right_iris=blobs.GetBlob(brightest->b);

			int count = blobs.GetNumBlobs();


			if(right_iris.MinX() > left_iris.MinX()) //sort w.r.t. the minX //be careful. right_iris is shown left on the image.
			{
				tmp = left_iris;
				left_iris = right_iris;
				right_iris = tmp;
			}
			//save pupil position
			cam->left_eye_pupil.x = (left_iris.MaxX()+left_iris.MinX()) /2.0 ;
			cam->left_eye_pupil.y = (left_iris.MaxY()+left_iris.MinY()) /2.0;
			cam->left_eye_pupil.y-= 0;
			cam->right_eye_pupil.x = (right_iris.MaxX()+right_iris.MinX()) /2.0 ;
			cam->right_eye_pupil.y = (right_iris.MaxY()+right_iris.MinY()) /2.0;

			cam->right_eye_pupil.y-= 0 ;
			//The ROI is set around the detected pupil, approximating the size of the eye. i.e. the area where glints can be
			//Set the ROI for left eye

			pupilwidth = left_iris.MaxX()-left_iris.MinX();
			pupilheight = left_iris.MaxY()-left_iris.MinY();


#if POVRAYIMG
			pupilheight=max(pupilheight,pupilwidth);
			pupilwidth=max(pupilheight,pupilwidth);
			cam->left_eye_ROI.x = left_iris.MinX() - 2*pupilwidth;
			cam->left_eye_ROI.y = left_iris.MinY() - 2*pupilheight;
			cam->left_eye_ROI.width = 5*pupilwidth;
			cam->left_eye_ROI.height = 5*pupilheight;
#else
			cam->left_eye_ROI.x = left_iris.MinX() - 2.5*pupilwidth;
			cam->left_eye_ROI.y = left_iris.MinY() - 1*pupilheight;
			cam->left_eye_ROI.width = 6*pupilwidth;
			cam->left_eye_ROI.height = 3*pupilheight;
#endif

			//bounds check
			set_between(cam->left_eye_ROI.x,0,thresh_small->width);
			set_between(cam->left_eye_ROI.y,0,thresh_small->height);
			set_between(cam->left_eye_ROI.height,0,thresh_small->height - cam->left_eye_ROI.y);
			set_between(cam->left_eye_ROI.width,0,thresh_small->width - cam->left_eye_ROI.x);
			cam->left_eye_ROI.width &= -4;//iplimage width has to be a multiple of 4 

			//Set the ROI for right eye
			pupilwidth = right_iris.MaxX()-right_iris.MinX();
			pupilheight = right_iris.MaxY()-right_iris.MinY();

#if POVRAYIMG
			pupilheight=max(pupilheight,pupilwidth);
			pupilwidth=max(pupilheight,pupilwidth);

			cam->right_eye_ROI.x = right_iris.MinX() - 2*pupilwidth;
			cam->right_eye_ROI.y = right_iris.MinY() - 2*pupilheight;
			cam->right_eye_ROI.width = 5*pupilwidth; 
			cam->right_eye_ROI.height = 5*pupilheight;
#else

			cam->right_eye_ROI.x = right_iris.MinX() - 2.5*pupilwidth;
			cam->right_eye_ROI.y = right_iris.MinY() - 1*pupilheight;
			cam->right_eye_ROI.width = 6*pupilwidth; 
			cam->right_eye_ROI.height = 3*pupilheight;
#endif				

			//bounds check
			set_between(cam->right_eye_ROI.x,0,thresh_small->width);
			set_between(cam->right_eye_ROI.y,0,thresh_small->height);
			set_between(cam->right_eye_ROI.height,0,thresh_small->height - cam->right_eye_ROI.y);
			set_between(cam->right_eye_ROI.width,0,thresh_small->width - cam->right_eye_ROI.x);
			cam->right_eye_ROI.width &= -4;//iplimage width has to be a multiple of 4

			//map the coordinates to old size (i.e. foo= 2*foo)
			cam->left_eye_pupil.x *=2;
			cam->left_eye_pupil.y *=2;

			cam->left_eye_ROI.x *=2;
			cam->left_eye_ROI.y *=2;
			cam->left_eye_ROI.width *=2;
			cam->left_eye_ROI.height*=2;

			cam->right_eye_pupil.x *=2;
			cam->right_eye_pupil.y *=2;

			cam->right_eye_ROI.x *=2;
			cam->right_eye_ROI.y *=2;
			cam->right_eye_ROI.width *=2;
			cam->right_eye_ROI.height *=2;

			//second step:
#if !POVRAYIMG
			CvBox2D left_center,right_center;
			memset(&left_center,0,sizeof(CvBox2D));
			memset(&right_center,0,sizeof(CvBox2D));

			////Search for the pupil inside the ROI
			double thresh=0.96;
			cvSetImageROI(cam->DarkFrame,cam->left_eye_ROI);
			IplImage *ROI_left = cvCreateImage(cvSize(cam->left_eye_ROI.width,cam->left_eye_ROI.height),IPL_DEPTH_8U,1);
			cvCopy(cam->DarkFrame,ROI_left);

			if (find_pupil_pos(ROI_left,left_center) >0)
			{//and update the ROI
				cam->left_eye_pupil.x = left_center.center.x + cam->left_eye_ROI.x;
				cam->left_eye_pupil.y = left_center.center.y + cam->left_eye_ROI.y;
			}



			cvReleaseImage(&ROI_left);
			ROI_left=NULL;

			//	do the same for the right eye
			cvSetImageROI(cam->DarkFrame,cam->right_eye_ROI);
			IplImage *ROI_right = cvCreateImage(cvSize(cam->right_eye_ROI.width,cam->right_eye_ROI.height),IPL_DEPTH_8U,1);
			cvCopy(cam->DarkFrame,ROI_right);
			if( find_pupil_pos(ROI_right,right_center) > 0 )
			{
				cam->right_eye_pupil.x = right_center.center.x + cam->right_eye_ROI.x;
				cam->right_eye_pupil.y = right_center.center.y + cam->right_eye_ROI.y; 
			}
			//if (_camnumber==1) cvShowImage("dbg2",ROI_right);
			

			cvReleaseImage(&ROI_right);
			ROI_right=NULL;
#endif
			//	cvResetImageROI(thresh);
			cvResetImageROI(cam->DarkFrame);
		}

#ifdef _DEBUG				
		cvRectangle( thresh, cvPoint(cam->left_eye_ROI.x , cam->left_eye_ROI.y ), cvPoint ( cam->left_eye_ROI.x + cam->left_eye_ROI.width, cam->left_eye_ROI.y + cam->left_eye_ROI.height), CV_RGB(250, 250, 250));
		cvRectangle( thresh, cvPoint(cam->right_eye_ROI.x , cam->right_eye_ROI.y ), cvPoint ( cam->right_eye_ROI.x + cam->right_eye_ROI.width, cam->right_eye_ROI.y + cam->right_eye_ROI.height), CV_RGB(250, 250, 250));
		DrawCross(_image,cam->left_eye_pupil);
		DrawCross(_image,cam->right_eye_pupil);
		cvShowImage((prefix + "thresh_pupil").c_str(),thresh);	
#endif


		cvSaveImage((write_prefix + "thresh_diff.jpg").c_str(),thresh);

		cvReleaseImage(&diff);
		diff=NULL;
		cvReleaseImage(&thresh_small);
		thresh_small=NULL;
		cvReleaseImage(&thresh);
		thresh=NULL;



	}


	if(
		(detect_glint)&&
		(cam->left_eye_ROI.width + cam->left_eye_ROI.x<= cam->DarkFrame->width)&&(cam->left_eye_ROI.width >=0)&&
		(cam->left_eye_ROI.height + cam->left_eye_ROI.y <= cam->DarkFrame->height)&&(cam->left_eye_ROI.height >=0)&&
		(cam->right_eye_ROI.width + cam->right_eye_ROI.x <= cam->DarkFrame->width)&&(cam->right_eye_ROI.width >=0)&&
		(cam->right_eye_ROI.height + cam->right_eye_ROI.y <= cam->DarkFrame->height)&&(cam->right_eye_ROI.height >=0)
		)

	{
		IplImage *left_eye_image = cvCreateImage(cvSize(cam->left_eye_ROI.width,cam->left_eye_ROI.height),IPL_DEPTH_8U,1);
		cvSetImageROI(cam->DarkFrame,cam->left_eye_ROI);
		cvCopy(cam->DarkFrame,left_eye_image);
		cvResetImageROI(cam->DarkFrame);

		IplImage *right_eye_image = cvCreateImage(cvSize(cam->right_eye_ROI.width,cam->right_eye_ROI.height),IPL_DEPTH_8U,1);
		cvSetImageROI(cam->DarkFrame,cam->right_eye_ROI);
		cvCopy(cam->DarkFrame,right_eye_image);
		cvResetImageROI(cam->DarkFrame);

		cvSaveImage((write_prefix + "right_ROI.jpg").c_str(),right_eye_image);


#if POVRAYIMG

		adaptiveThreshold(right_eye_image,right_eye_image,0.9999);
		adaptiveThreshold(left_eye_image,left_eye_image,0.9999);
#else
		cvCanny(right_eye_image,right_eye_image,100,200);
		cvCanny(left_eye_image,left_eye_image,100,200);
#endif
		cvSaveImage((write_prefix + "right_ROI_canny.jpg").c_str(),right_eye_image);


#ifdef _DEBUG		

		cvShowImage((prefix+"ROI_right").c_str(),right_eye_image);
		cvShowImage((prefix+"ROI_left").c_str(),left_eye_image);

#endif
		int blob_threshold=10;//mininum blobbrightness

#if POVRAYIMG
		int max_blob_size=200;//maximum blobsize
#else
		int max_blob_size=80;//maximum blobsize
#endif
		int min_blob_size=1;
		CBlobResult blobs;
		blobs = CBlobResult(right_eye_image,NULL,blob_threshold,true);

		sprintf_s(buffer,"%d",_camnumber);
		std::string cam_prefix = std::string(buffer);

		//blobs.PrintBlobs((char*)(cam_prefix + "rblobs.txt").c_str());

		// discard some blobs which can't be glints
		/*blobs.Filter(blobs,B_INCLUDE, CBlobGetArea(), B_GREATER,  2);*/
		blobs.Filter(blobs,B_EXCLUDE,CBlobGetArea(),B_GREATER,max_blob_size);
		blobs.Filter(blobs,B_EXCLUDE,CBlobGetExterior(),B_GREATER,0);
		blobs.Filter(blobs,B_EXCLUDE,CBlobGetParent(),B_GREATER,1);
		//// create a file with filtered results
		//blobs.PrintBlobs((char*)(write_prefix + "filtered_blobs.txt").c_str());


		//take the glints nearest to the pupil center
		//map the blobs to the glints. we assume they are ordered
		//from left to right(in the image)
		//left = LED1
		//middle = LED2
		//middle = LED0

#if POVRAYIMG
		/*		while(blobs.GetNumBlobs()>=4)
		blobs.Filter( blobs, B_EXCLUDE, CBlobGetArea(), B_LESS, min_blob_size++ );
		*/
#endif 
		if(blobs.GetNumBlobs()>=1)
		{
			CBlob left,middle,right,a,b,c;

			//take the 3 blobs next to the pupilcenter
			blobs.GetNthBlob(CBlobGetDistanceFromPoint(cam->right_eye_pupil.x - cam->right_eye_ROI.x,cam->right_eye_pupil.y - cam->right_eye_ROI.y),blobs.GetNumBlobs()-1,a);
			blobs.GetNthBlob(CBlobGetDistanceFromPoint(cam->right_eye_pupil.x - cam->right_eye_ROI.x,cam->right_eye_pupil.y - cam->right_eye_ROI.y),blobs.GetNumBlobs()-2,b);
			blobs.GetNthBlob(CBlobGetDistanceFromPoint(cam->right_eye_pupil.x - cam->right_eye_ROI.x,cam->right_eye_pupil.y - cam->right_eye_ROI.y),blobs.GetNumBlobs()-3,c);

			//sort them by their x value
			if( (a.MinX()<=b.MinX()) &&  (b.MinX()<=c.MinX()))			
			{
				left=a; middle=b; right=c;
			}
			else if ( (a.MinX()<=c.MinX()) &&  (c.MinX()<=b.MinX()))
			{
				left=a;middle=c;right=b;
			}
			else if ( (b.MinX()<=a.MinX()) &&  (a.MinX()<=c.MinX()))
			{
				left=b;middle=a;right=c;
			}
			else if ( (b.MinX()<=c.MinX()) &&  (c.MinX()<=a.MinX()))
			{
				left=b;middle=c;right=a;
			}
			else if ( (c.MinX()<=a.MinX()) &&  (a.MinX()<=b.MinX()))
			{
				left=c;middle=a;right=b;
			}
			else if ( (c.MinX()<=b.MinX()) &&  (b.MinX()<=a.MinX()))
			{
				left=c;middle=b;right=a;
			}

			//if we have old planes stored, delete them
			if (cam->right_eye_planes.size() !=0) cam->right_eye_planes.clear();
			//calculate new planes and store them
			double glintx = ((left.MinX() + left.MaxX())/2.0) + cam->right_eye_ROI.x;
			double glinty = ((left.MinY() + left.MaxY())/2.0) + cam->right_eye_ROI.y;
			add_plane(*cam,cvPoint2D64f(glintx,glinty),cams_[1].LED1_position,cam->right_eye_planes);

			cvCircle(_image,cvPoint(glintx,glinty),5,CV_RGB(255,255,255));

			glintx = ((middle.MinX() + middle.MaxX())/2.0) + cam->right_eye_ROI.x;
			glinty = ((middle.MinY() + middle.MaxY())/2.0) + cam->right_eye_ROI.y;
			add_plane(*cam,cvPoint2D64f(glintx,glinty),cams_[2].LED1_position,cam->right_eye_planes);

			cvCircle(_image,cvPoint(glintx,glinty),5,CV_RGB(255,255,255));


			glintx = ((right.MinX() + right.MaxX())/2.0) + cam->right_eye_ROI.x;
			glinty = ((right.MinY() + right.MaxY())/2.0) + cam->right_eye_ROI.y;
			add_plane(*cam,cvPoint2D64f(glintx,glinty),cams_[0].LED1_position,cam->right_eye_planes);

			cvCircle(_image,cvPoint(glintx,glinty),5,CV_RGB(255,255,255));

		}

		//and now the same for the left eye
		blobs = CBlobResult(left_eye_image,NULL,blob_threshold,true);
		//blobs.PrintBlobs( "blobs.txt" );

		min_blob_size=1;
		// discard some blobs which can't be glints
		blobs.Filter(blobs,B_EXCLUDE,CBlobGetArea(),B_GREATER,max_blob_size);
		blobs.Filter(blobs,B_EXCLUDE,CBlobGetExterior(),B_GREATER,0);
		blobs.Filter(blobs,B_EXCLUDE,CBlobGetParent(),B_GREATER,1);
		//// create a file with filtered results
		//blobs.PrintBlobs( "filteredBlobs.txt" );

#if POVRAYIMG
		while(blobs.GetNumBlobs()>=4)
			blobs.Filter( blobs, B_EXCLUDE, CBlobGetArea(), B_LESS, min_blob_size++ );
#endif 
		if(blobs.GetNumBlobs()>=1)
		{
			CBlob left,middle,right,a,b,c;

			//take the 3 blobs next to the pupilcenter
			blobs.GetNthBlob(CBlobGetDistanceFromPoint(cam->left_eye_pupil.x - cam->left_eye_ROI.x,cam->left_eye_pupil.y - cam->left_eye_ROI.y),blobs.GetNumBlobs()-1,a);
			blobs.GetNthBlob(CBlobGetDistanceFromPoint(cam->left_eye_pupil.x - cam->left_eye_ROI.x,cam->left_eye_pupil.y - cam->left_eye_ROI.y),blobs.GetNumBlobs()-2,b);
			blobs.GetNthBlob(CBlobGetDistanceFromPoint(cam->left_eye_pupil.x - cam->left_eye_ROI.x,cam->left_eye_pupil.y - cam->left_eye_ROI.y),blobs.GetNumBlobs()-3,c);

			//sort them by their x value
			if( (a.MinX()<=b.MinX()) &&  (b.MinX()<=c.MinX()))			
			{
				left=a; middle=b; right=c;
			}
			else if ( (a.MinX()<=c.MinX()) &&  (c.MinX()<=b.MinX()))
			{
				left=a;middle=c;right=b;
			}
			else if ( (b.MinX()<=a.MinX()) &&  (a.MinX()<=c.MinX()))
			{
				left=b;middle=a;right=c;
			}
			else if ( (b.MinX()<=c.MinX()) &&  (c.MinX()<=a.MinX()))
			{
				left=b;middle=c;right=a;
			}
			else if ( (c.MinX()<=a.MinX()) &&  (a.MinX()<=b.MinX()))
			{
				left=c;middle=a;right=b;
			}
			else if ( (c.MinX()<=b.MinX()) &&  (b.MinX()<=a.MinX()))
			{
				left=c;middle=b;right=a;
			}

			//if we have old planes stored, delete them
			if (cam->left_eye_planes.size() !=0) cam->left_eye_planes.clear();
			//calculate new planes and store them
			double glintx = ((left.MinX() + left.MaxX())/2.0) + cam->left_eye_ROI.x;
			double glinty = ((left.MinY() + left.MaxY())/2.0) + cam->left_eye_ROI.y;
			add_plane(*cam,cvPoint2D64f(glintx,glinty),cams_[1].LED1_position,cam->left_eye_planes,1);

			cvCircle(_image,cvPoint(glintx,glinty),5,CV_RGB(255,255,255));

			glintx = ((middle.MinX() + middle.MaxX())/2.0) + cam->left_eye_ROI.x;
			glinty = ((middle.MinY() + middle.MaxY())/2.0) + cam->left_eye_ROI.y;
			add_plane(*cam,cvPoint2D64f(glintx,glinty),cams_[2].LED1_position,cam->left_eye_planes,2);

			cvCircle(_image,cvPoint(glintx,glinty),5,CV_RGB(255,255,255));


			glintx = ((right.MinX() + right.MaxX())/2.0) + cam->left_eye_ROI.x;
			glinty = ((right.MinY() + right.MaxY())/2.0) + cam->left_eye_ROI.y;
			add_plane(*cam,cvPoint2D64f(glintx,glinty),cams_[0].LED1_position,cam->left_eye_planes,3);

			cvCircle(_image,cvPoint(glintx,glinty),5,CV_RGB(255,255,255));

		}
#ifdef _DEBUG				
		cvRectangle(_image, cvPoint(cam->left_eye_ROI.x , cam->left_eye_ROI.y ), cvPoint ( cam->left_eye_ROI.x + cam->left_eye_ROI.width, cam->left_eye_ROI.y + cam->left_eye_ROI.height), CV_RGB(250, 250, 250));
		cvRectangle(_image, cvPoint(cam->right_eye_ROI.x , cam->right_eye_ROI.y ), cvPoint ( cam->right_eye_ROI.x + cam->right_eye_ROI.width, cam->right_eye_ROI.y + cam->right_eye_ROI.height), CV_RGB(250, 250, 250));
#endif

		cvReleaseImage(&left_eye_image);
		left_eye_image=NULL;
		cvReleaseImage(&right_eye_image);
		right_eye_image=NULL;

	}

}
