#include "StdAfx.h"
#include "Table.h"
#include "GTCamera.h"
#include "algebra.h"

Table::Table(void)
{
}

Table::~Table(void)
{
}

//loads the 3D positions of the table
void Table::load_position_data(char *_filename)
{

	//load table data
	FILE *fp = fopen(_filename,"r");

	//read and load table position
	fscanf(fp,"%lf", &(p1.x));
	fscanf(fp,"%lf", &(p1.y));
	fscanf(fp,"%lf", &(p1.z));
	
	fscanf(fp,"%lf", &(p2.x));
	fscanf(fp,"%lf", &(p2.y));
	fscanf(fp,"%lf", &(p2.z));

	fscanf(fp,"%lf", &(p3.x));
	fscanf(fp,"%lf", &(p3.y));
	fscanf(fp,"%lf", &(p3.z));

	fclose(fp);
}

void Table::calc_gaze_points(CvPoint3D64f &_gaze,CvPoint3D64f &_origin)
{

	plane table_plane;
	CvPoint3D64f xdir,ydir,normal,corner_to_intersect;//xdir= direction vector from P1 to P2, ydir = P1 to p3;normal is the normal of the table plane
	//corner_to_intersect is the direction vector from the top left corner to the intersection point.
	xdir.x = p2.x - p1.x;
	xdir.y = p2.y - p1.y;
	xdir.z = p2.z - p1.z;

	ydir.x = p3.x - p1.x;
	ydir.y = p3.y - p1.y;
	ydir.z = p3.z - p1.z;
	
	CrossProduct(xdir,ydir,normal);

	Normalize(normal);
	table_plane.n_x = normal.x;
	table_plane.n_y = normal.y;
	table_plane.n_z = normal.z;
	table_plane.d = -(ScalarProduct(normal,p1));

	//see the thesis for more info
	double lambda = -(table_plane.d + ScalarProduct(normal,_origin)) ;
	lambda /= ScalarProduct(normal,_gaze);

	CvPoint3D64f gazepoint;// the 3d intersection of the gaze ray and the display plane
	gazepoint.x = _origin.x + lambda*_gaze.x;
	gazepoint.y = _origin.y + lambda*_gaze.y;
	gazepoint.z = _origin.z + lambda*_gaze.z;
	corner_to_intersect.x = gazepoint.x - p1.x;
	corner_to_intersect.y = gazepoint.y - p1.y;
	corner_to_intersect.z = gazepoint.z - p1.z;

	//calc 2d pos in virtual values
	//e.g. (0.4,0.6) means that the point is at p1 + 0.4*xdir + 0.6*ydir
	this->impactpoint.x = ScalarProduct(xdir,corner_to_intersect);
	this->impactpoint.x /= ScalarProduct(xdir,xdir);

	this->impactpoint.y = ScalarProduct(ydir,corner_to_intersect);
	this->impactpoint.y /= ScalarProduct(ydir,ydir);

}