#pragma once
#include <stdafx.h>
#include <math.h>
#include <memory.h>
#include <time.h>
#include "algebra.h"
#include <list>



//calculates determinant for linsolve //TODO consistent ordering (columnwiser or rowwise)
double finddet(double a1,double a2, double a3,double b1, double b2,double b3, double c1, double c2, double c3)
{
	return ((a1*b2*c3)-(a1*b3*c2)-(a2*b1*c3)+(a3*b1*c2)+(a2*b3*c1)-(a3*b2*c1)); /*expansion of a 3x3 determinant*/
}

//uses cramer's rule to get the solution of a 3x3 LES
bool linsolve(double **A,double *l)
{

	double	 det=finddet(A[0][0],A[1][0],A[2][0],A[0][1],A[1][1],A[2][1],A[0][2],A[1][2],A[2][2]);   /*Find determinants*/

	double	 detx=finddet(l[0],l[1],l[2],A[0][1],A[1][1],A[2][1],A[0][2],A[1][2],A[2][2]);
	double   dety=finddet(A[0][0],A[1][0],A[2][0],l[0],l[1],l[2],A[0][2],A[1][2],A[2][2]);
	double   detz=finddet(A[0][0],A[1][0],A[2][0],A[0][1],A[1][1],A[2][1],l[0],l[1],l[2]);

	if(det!=0)
	{
		l[0] = detx/det;
		l[1] = dety/det;
		l[2] = detz/det;
		return true;
	}
	else return false;

}

//solves LES with cholewsky decomposition
void CrossProduct(CvPoint3D64f &_vec1,CvPoint3D64f &_vec2,CvPoint3D64f &_normal)
{
	_normal.x = _vec1.y*_vec2.z - _vec1.z*_vec2.y;
	_normal.y = _vec1.z*_vec2.x - _vec1.x*_vec2.z;
	_normal.z = _vec1.x*_vec2.y - _vec1.y*_vec2.x;

}

double ScalarProduct(CvPoint3D64f &_vec1,CvPoint3D64f &_vec2)
{
	return (_vec1.x*_vec2.x + _vec1.y*_vec2.y + _vec1.z*_vec2.z);

}
//see my thesis in section "cornea center detection for real data" for explanation of what is going on.
//returns false if det = 0 i.e. not solvable
bool point_nearest_to_planes(plane *_planes,int _numplanes, CvPoint3D64f &_point)
{
	// 3 by 3 matrix. stores a part of the quadric
	double **A=(double **)malloc(sizeof(double*)*3);
	for(int i=0;i < 3;i++)
	{
		A[i] = (double*)malloc(sizeof(double)*3);
		memset(A[i],0,sizeof(double)*3);
	}

	// 1 by 3 vector. stores a part of the quadric
	double *l=(double*)malloc(sizeof(double)*3);
	memset(l,0,sizeof(double)*3);
	//for each plane: calculate the quadric and sum them up in A and -l
	for(int i=0; i < _numplanes;i++)
	{
		A[0][0]+=_planes[i].n_x*_planes[i].n_x;
		A[0][1]+=_planes[i].n_x*_planes[i].n_y;
		A[0][2]+=_planes[i].n_x*_planes[i].n_z;
		l[0]-=_planes[i].n_x*_planes[i].d;

		A[1][0]+=_planes[i].n_y*_planes[i].n_x;
		A[1][1]+=_planes[i].n_y*_planes[i].n_y;
		A[1][2]+=_planes[i].n_y*_planes[i].n_z;
		l[1]-=_planes[i].n_y*_planes[i].d;

		A[2][0]+=_planes[i].n_x*_planes[i].n_z;
		A[2][1]+=_planes[i].n_y*_planes[i].n_z;
		A[2][2]+=_planes[i].n_z*_planes[i].n_z;
		l[2]-=_planes[i].n_z*_planes[i].d;
	}
	//solve the LES
	bool solvable = linsolve(A,l);

	/*int idx[3];
	if(solvable)
	{
		ludcmp(A,3,idx);
		lubksb(A,3,idx,l);
	}*/
	//store the solution
	_point.x = l[0];
	_point.y = l[1];
	_point.z = l[2];

	for(int i=0;i < 3;i++)
	{
		free(A[i]);
	}
	free(A);
	free(l);
	if (solvable) return true;
	else return false;
}

void unproject(GTCamera &_cam,CvPoint2D64f &_glint,CvPoint3D64f &_cam_to_glint)
{
	_cam_to_glint.x = (_glint.x - _cam.cx)/ _cam.fx;
	_cam_to_glint.y = (_cam.cy - _glint.y )/ _cam.fy;
	_cam_to_glint.z = 1;

	//map from the local coordinate system to the global coordinate system x'=R^(-1)*x;
	double x,y,z;
	x = _cam_to_glint.x * _cam.rotation[0][0] + _cam_to_glint.y * _cam.rotation[0][1] + _cam_to_glint.z * _cam.rotation[0][2];
	y = _cam_to_glint.x * _cam.rotation[1][0] + _cam_to_glint.y * _cam.rotation[1][1] + _cam_to_glint.z * _cam.rotation[1][2];
	z = _cam_to_glint.x * _cam.rotation[2][0] + _cam_to_glint.y * _cam.rotation[2][1] + _cam_to_glint.z * _cam.rotation[2][2];
	_cam_to_glint.x = x;
	_cam_to_glint.y = y;
	_cam_to_glint.z = z;

	Normalize(_cam_to_glint);
}
//calculates the plane from the given points on it
// and adds it to _planes
// see my thesis in section "calculate the cornea center" for details
void add_plane(GTCamera &_cam, CvPoint2D64f &_glint, CvPoint3D64f &_light, std::vector<plane> &_planes, int _store_at)
{
	CvPoint3D64f cam_to_glint = cvPoint3D64f(0,0,0);
	CvPoint3D64f cam_to_light = cvPoint3D64f(0,0,0);
	CvPoint3D64f normal = cvPoint3D64f(0,0,0);

	unproject(_cam,_glint,cam_to_glint);

	cam_to_light.x =_light.x - _cam.camera_position.x;
	cam_to_light.y =_light.y - _cam.camera_position.y;
	cam_to_light.z =_light.z - _cam.camera_position.z;

	Normalize(cam_to_light);
	
	
	_planes.resize(_planes.size()+1);
	CrossProduct(cam_to_glint,cam_to_light,normal);
	Normalize(normal);
	//calculate the normal via crossproduct of CG and CL
	_planes.back().n_x = normal.x;
	_planes.back().n_y = normal.y;
	_planes.back().n_z = normal.z;
	_planes.back().d = -ScalarProduct(normal,_cam.camera_position);

	if (_store_at==1)
	{
		_cam.cam_to_glint1.x=cam_to_glint.x;
		_cam.cam_to_glint1.y=cam_to_glint.y;
		_cam.cam_to_glint1.z=cam_to_glint.z;
	}
	if (_store_at==2)
	{
		_cam.cam_to_glint2.x=cam_to_glint.x;
		_cam.cam_to_glint2.y=cam_to_glint.y;
		_cam.cam_to_glint2.z=cam_to_glint.z;
	}
	if (_store_at==3)
	{
		_cam.cam_to_glint3.x=cam_to_glint.x;
		_cam.cam_to_glint3.y=cam_to_glint.y;
		_cam.cam_to_glint3.z=cam_to_glint.z;
	}
	if (_store_at==4)
	{
		_cam.cam_to_pupil.x=cam_to_glint.x;
		_cam.cam_to_pupil.y=cam_to_glint.y;
		_cam.cam_to_pupil.z=cam_to_glint.z;
	}

}

//calculates the determinant of a 3x3 matrix
double det(double **_mat_a)
{
	double detA=0;
	detA+=_mat_a[0][0]*_mat_a[1][1]*_mat_a[2][2]+
		_mat_a[0][1]*_mat_a[1][2]*_mat_a[2][0] +
		_mat_a[0][2]*_mat_a[1][0]*_mat_a[2][1] -
		_mat_a[0][2]*_mat_a[1][1]*_mat_a[2][0] -
		_mat_a[0][0]*_mat_a[1][2]*_mat_a[2][1] -
		_mat_a[0][1]*_mat_a[1][0]*_mat_a[2][2];
	return detA;
}

#define A _mat_a[0][0]
#define B _mat_a[0][1]
#define C _mat_a[0][2]

#define D _mat_a[1][0]
#define E _mat_a[1][1]
#define F _mat_a[1][2]

#define G _mat_a[2][0]
#define H _mat_a[2][1]
#define I _mat_a[2][2]

//inverts _mat_a \in R^(3x3)
bool invert_matrix(double **_mat_a,double **_invert)
{
	double detA = det(_mat_a);
	if (detA!=0)
	{
		_invert[0][0] = ((E*I) - (F*H))/detA;
		_invert[0][1] = ((C*H) - (B*I))/detA;
		_invert[0][2] = ((B*F) - (C*E))/detA;

		_invert[1][0] = ((F*G) - (D*I))/detA;
		_invert[1][1] = ((A*I) - (C*G))/detA;
		_invert[1][2] = ((C*D) - (A*F))/detA;

		_invert[2][0] = ((D*H) - (E*G))/detA;
		_invert[2][1] = ((B*G) - (A*H))/detA;
		_invert[2][2] = ((A*E) - (B*D))/detA;


		return true;
	}
	else return false;

}

CvPoint3D64f  GTCamera::right_eye_center = cvPoint3D64f(0,0,0);
CvPoint3D64f  GTCamera::right_eye_gaze = cvPoint3D64f(0,0,0);
CvPoint3D64f  GTCamera::left_eye_center = cvPoint3D64f(0,0,0);
CvPoint3D64f  GTCamera::left_eye_gaze = cvPoint3D64f(0,0,0);
CvPoint3D64f  GTCamera::gaze_focus = cvPoint3D64f(0,0,0);
std::vector<plane> GTCamera::gazeplanes;
CvPoint3D64f  GTCamera::non_modified_gaze_focus = cvPoint3D64f(0,0,0);
#if POVRAYIMG
CvPoint3D64f  GTCamera::reference_gaze_focus = cvPoint3D64f(0,0,0);

#endif
//calculate the gaze direction for each eye.

void calc_gaze( std::vector<GTCamera> &_cams,Table& _table)
{

	GTCamera::gazeplanes.clear();
	std::vector<GTCamera>::iterator cam_it;

	int i=0;
	for(cam_it=_cams.begin();cam_it!=_cams.end();cam_it++)
	{
		/*if( (i==0)||(i==1))*/ add_plane(*cam_it,cam_it->left_eye_pupil,GTCamera::left_eye_center,GTCamera::gazeplanes,4);
		/*if(i!=2)*/ add_plane(*cam_it,cam_it->right_eye_pupil,GTCamera::right_eye_center,GTCamera::gazeplanes);
		i++;
	}

	point_nearest_to_planes(&GTCamera::gazeplanes[0],GTCamera::gazeplanes.size(),GTCamera::gaze_focus);
	GTCamera::right_eye_gaze.x=GTCamera::gaze_focus.x - GTCamera::right_eye_center.x;
	GTCamera::right_eye_gaze.y=GTCamera::gaze_focus.y - GTCamera::right_eye_center.y;
	GTCamera::right_eye_gaze.z=GTCamera::gaze_focus.z - GTCamera::right_eye_center.z;
	GTCamera::left_eye_gaze.x=GTCamera::gaze_focus.x - GTCamera::left_eye_center.x;
	GTCamera::left_eye_gaze.y=GTCamera::gaze_focus.y - GTCamera::left_eye_center.y;
	GTCamera::left_eye_gaze.z=GTCamera::gaze_focus.z - GTCamera::left_eye_center.z;
	//calculate the intersection of the (two-eye) gaze direction and the table.

	CvPoint3D64f origin,gaze;
	origin.x = (GTCamera::right_eye_center.x + GTCamera::left_eye_center.x)/2.0;
	origin.y = (GTCamera::right_eye_center.y + GTCamera::left_eye_center.y)/2.0;
	origin.z = (GTCamera::right_eye_center.z + GTCamera::left_eye_center.z)/2.0;

	gaze.x  = GTCamera::gaze_focus.x - origin.x;
	gaze.y  = GTCamera::gaze_focus.y - origin.y;
	gaze.z  = GTCamera::gaze_focus.z - origin.z;

	_table.calc_gaze_points(gaze,origin);

}
//calculates A*B=out
void matrix_mul(double **_A,double **_B, double **_out)
{
	double tmp[3][3];//need for the case that A==out or B==out
	tmp[0][0]=_A[0][0]*_B[0][0] + _A[0][1]*_B[1][0] + _A[0][2]*_B[2][0];
	tmp[0][1]=_A[0][0]*_B[0][1] + _A[0][1]*_B[1][1] + _A[0][2]*_B[2][1];
	tmp[0][2]=_A[0][0]*_B[0][2] + _A[0][1]*_B[1][2] + _A[0][2]*_B[2][2];

	tmp[1][0]=_A[1][0]*_B[0][0] + _A[1][1]*_B[1][0] + _A[1][2]*_B[2][0];
	tmp[1][1]=_A[1][0]*_B[0][1] + _A[1][1]*_B[1][1] + _A[1][2]*_B[2][1];
	tmp[1][2]=_A[1][0]*_B[0][2] + _A[1][1]*_B[1][2] + _A[1][2]*_B[2][2];

	tmp[2][0]=_A[2][0]*_B[0][0] + _A[2][1]*_B[1][0] + _A[2][2]*_B[2][0];
	tmp[2][1]=_A[2][0]*_B[0][1] + _A[2][1]*_B[1][1] + _A[2][2]*_B[2][1];
	tmp[2][2]=_A[2][0]*_B[0][2] + _A[2][1]*_B[1][2] + _A[2][2]*_B[2][2];

	for(int i=0;i<3;i++)
	{
		memcpy(_out[i],tmp[i],sizeof(double)*3);
	}


}

//calculates righthand-sided rotationmatrix
//if _invert_rotation is true, then the matrix is initialized with a z flip
//i.e. R_z*R_y*R_x (*p)
void RotationMatrix(double x_angle,double y_angle,double z_angle,bool _invert_rotation, double** _matrix)
{
	double **rotx,**roty,**rotz,**tmp;

	rotx = (double **)malloc(sizeof(double)*3);
	for(int i=0;i<3;i++)
	{
		rotx[i]=(double*)malloc(sizeof(double)*3);
		memset(rotx[i],0,sizeof(double)*3);
	}
	roty = (double **)malloc(sizeof(double)*3);
	for(int i=0;i<3;i++)
	{
		roty[i]=(double*)malloc(sizeof(double)*3);
		memset(roty[i],0,sizeof(double)*3);
	}
	rotz = (double **)malloc(sizeof(double)*3);
	for(int i=0;i<3;i++)
	{
		rotz[i]=(double*)malloc(sizeof(double)*3);
		memset(rotz[i],0,sizeof(double)*3);
	}
	tmp = (double **)malloc(sizeof(double)*3);
	for(int i=0;i<3;i++)
	{
		tmp[i]=(double*)malloc(sizeof(double)*3);
		memset(tmp[i],0,sizeof(double)*3);
	}
	//convert to radian
	x_angle*= 3.14159265358979323846/180.0;
	y_angle*= 3.14159265358979323846/180.0;
	z_angle*= 3.14159265358979323846/180.0;
	if(_invert_rotation)
	{
		rotx[0][0] = 1;
		rotx[0][1] = 0;
		rotx[0][2] = 0;

		rotx[1][0] = 0;
		rotx[1][1] = cos(x_angle);
		rotx[1][2] = sin(x_angle);

		rotx[2][0] = 0;
		rotx[2][1] = sin(x_angle);
		rotx[2][2] = -cos(x_angle);
	}
	else
	{

		rotx[0][0] = 1;
		rotx[0][1] = 0;
		rotx[0][2] = 0;

		rotx[1][0] = 0;
		rotx[1][1] = cos(x_angle);
		rotx[1][2] = -sin(x_angle);

		rotx[2][0] = 0;
		rotx[2][1] = sin(x_angle);
		rotx[2][2] = cos(x_angle);
	}


	roty[0][0] = cos(y_angle);
	roty[0][1] = 0;
	roty[0][2] = sin(y_angle);

	roty[1][0] = 0;
	roty[1][1] = 1;
	roty[1][2] = 0;

	roty[2][0] = -sin(y_angle);
	roty[2][1] = 0;
	roty[2][2] = cos(y_angle);



	rotz[0][0] = cos(z_angle);
	rotz[0][1] = -sin(z_angle);
	rotz[0][2] = 0;

	rotz[1][0] = sin(z_angle);
	rotz[1][1] = cos(z_angle);
	rotz[1][2] = 0;

	rotz[2][0] = 0;
	rotz[2][1] = 0;
	rotz[2][2] = 1;

	matrix_mul((double**)roty,(double**)rotx,(double**)tmp);
	matrix_mul((double**)rotz,(double**)tmp,(double**)_matrix);
	for(int i=0;i<3;i++)
	{
		free(rotx[i]);
	}
	free(rotx);
	for(int i=0;i<3;i++)
	{
		free(roty[i]);
	}
	free(roty);
	for(int i=0;i<3;i++)
	{
		free(rotz[i]);
	}
	free(rotz);
	for(int i=0;i<3;i++)
	{
		free(tmp[i]);
	}
	free(tmp);

}
//calculates rotation matrix around a given vector with _angle

void RotationMatrix(CvPoint3D64f &_rotate_around,double _angle, double **_matrix)
{

	double s,c;
	_angle*=3.14159265358979323846/180.0;//convert to radian
	s =sin(_angle);
	c =cos(_angle);

	_matrix[0][0]=c + (1-c)*_rotate_around.x*_rotate_around.x;
	_matrix[0][1]=(1-c)*_rotate_around.x*_rotate_around.y -s*_rotate_around.z;
	_matrix[0][2]=(1-c)*_rotate_around.x*_rotate_around.z + s*_rotate_around.y;

	_matrix[1][0]=(1-c)*_rotate_around.y*_rotate_around.x +s*_rotate_around.z;
	_matrix[1][1]=c + (1-c)*_rotate_around.y*_rotate_around.y;
	_matrix[1][2]=(1-c)*_rotate_around.y*_rotate_around.z - s*_rotate_around.x;

	_matrix[2][0]=(1-c)*_rotate_around.z*_rotate_around.x - s*_rotate_around.y;
	_matrix[2][1]=(1-c)*_rotate_around.z*_rotate_around.y + s*_rotate_around.x;
	_matrix[2][2]=c + (1-c)*_rotate_around.z*_rotate_around.z;




}

double calculateQuality(std::vector<GTCamera> &_cams,Table &_table)
{
	//calculate the reference and actual gazes
	CvPoint3D64f refpoint_on_display,actualpoint_on_display, reference_gaze,actual_gaze;

	actualpoint_on_display.x =(_table.impactpoint.x*(_table.p2.x - _table.p1.x)) + _table.p1.x;
	actualpoint_on_display.y= _table.p1.y - (_table.impactpoint.y*(_table.p1.y - _table.p3.y ) );
	actualpoint_on_display.z= _table.p1.z;

	reference_gaze.x = GTCamera::non_modified_gaze_focus.x - ((GTCamera::left_eye_center.x + GTCamera::right_eye_center.x)/2.0);
	reference_gaze.y = GTCamera::non_modified_gaze_focus.y - ((GTCamera::left_eye_center.y + GTCamera::right_eye_center.y)/2.0);
	reference_gaze.z = GTCamera::non_modified_gaze_focus.z - ((GTCamera::left_eye_center.z + GTCamera::right_eye_center.z)/2.0);

	actual_gaze.x = GTCamera::gaze_focus.x - ((GTCamera::left_eye_center.x + GTCamera::right_eye_center.x)/2.0);
	actual_gaze.y = GTCamera::gaze_focus.y - ((GTCamera::left_eye_center.y + GTCamera::right_eye_center.y)/2.0);
	actual_gaze.z = GTCamera::gaze_focus.z- ((GTCamera::left_eye_center.z + GTCamera::right_eye_center.z)/2.0);


	return (angle_between(reference_gaze,actual_gaze)) ;

}

double angle_between(CvPoint3D64f &_veca,CvPoint3D64f &_vecb)
{
	//calcute the cosinus of the angle between the two vectors via the scalar product
	double angle=_veca.x*_vecb.x + _veca.y*_vecb.y + _veca.z*_vecb.z;
	angle/=sqrt(pow(_veca.x,2)+pow(_veca.y,2)+pow(_veca.z,2));
	angle/=sqrt(pow(_vecb.x,2)+pow(_vecb.y,2)+pow(_vecb.z,2));

	//map to the angle and convert from radian to degrees
	return ( acos(angle)/3.14159265358979323846*180.0) ;
}

double random_flat(double _a, double _b)
{
	return ( (double) rand()/(double) RAND_MAX *(_b-_a) + _a);
}

double random_gauss(double _mean, double _dev)
{
//take a look at http://www.taygeta.com/random/gaussian.html for the explanation
		double x1, x2, w, y1, y2;
 
         do {
                 x1 = 2.0 * random_flat(0,1) - 1.0;
                 x2 = 2.0 * random_flat(0,1) - 1.0;
                 w = x1 * x1 + x2 * x2;
         } while ( w >= 1.0 );

		 w = sqrt( (-2.0 * log( w ) ) / w );
         y1 = x1 * w;
         y2 = x2 * w;
		 //ignore y2, it is only need for the generation of y1
		 //y1 and y2 are gauss distributed with mean 0 and sttdev 1 so just scale to the desired region
		 return y1*_dev+_mean;
}


void Normalize(CvPoint3D64f &_vec)
{
	double length=sqrt(pow(_vec.x,2)+pow(_vec.y,2)+pow(_vec.z,2));
	if (length!=0)
	{
		_vec.x/=length;
		_vec.y/=length;
		_vec.z/=length;
	}

}
void adaptive_calibration(std::vector<GTCamera> &_cams,std::vector<plane> &_left_eye_planes)
{
	std::vector<GTCamera>::iterator cam_it;

	int i=0;
	for(cam_it=_cams.begin();cam_it!=_cams.end();cam_it++)
	{

		//calc the ray from the camera to the eye using only the information of the camera itself. 
		//the ray is the average of the plane pair intersections.
		CvPoint3D64f cam_to_eye;

		CvPoint3D64f tmp1,tmp2,tmp3;//the plane intersections
		//assumes 3 glints per camera	
		CrossProduct(cvPoint3D64f(_left_eye_planes[i].n_x,_left_eye_planes[i].n_y,_left_eye_planes[i].n_z),
			cvPoint3D64f(_left_eye_planes[i+1].n_x,_left_eye_planes[i+1].n_y,_left_eye_planes[i+1].n_z),
			tmp1);
		Normalize(tmp1);
		CrossProduct(cvPoint3D64f(_left_eye_planes[i].n_x,_left_eye_planes[i].n_y,_left_eye_planes[i].n_z),
			cvPoint3D64f(_left_eye_planes[i+2].n_x,_left_eye_planes[i+2].n_y,_left_eye_planes[i+2].n_z),
			tmp2);
		Normalize(tmp2);
		CrossProduct(cvPoint3D64f(_left_eye_planes[i+1].n_x,_left_eye_planes[i+1].n_y,_left_eye_planes[i+1].n_z),
			cvPoint3D64f(_left_eye_planes[i+2].n_x,_left_eye_planes[i+2].n_y,_left_eye_planes[i+2].n_z),
			tmp3);
		Normalize(tmp3);
		CvPoint3D64f cam_to_eye_LES;//the ray from the camera to the eye_position generated by the LES approach
		cam_to_eye_LES.x= GTCamera::left_eye_center.x - cam_it->camera_position.x;
		cam_to_eye_LES.y= GTCamera::left_eye_center.y - cam_it->camera_position.y;
		cam_to_eye_LES.z= GTCamera::left_eye_center.z - cam_it->camera_position.z;

		Normalize(cam_to_eye_LES);
		//consistent orientation of the vectors
		//the should all point towards the eye
		if((angle_between(cam_to_eye_LES,tmp1) < -90)||(angle_between(cam_to_eye_LES,tmp1) > 90) )
		{
			tmp1.x*=-1;
			tmp1.y*=-1;
			tmp1.z*=-1;
		}
		if((angle_between(cam_to_eye_LES,tmp2) < -90)||(angle_between(cam_to_eye_LES,tmp2) > 90) )
		{
			tmp2.x*=-1;
			tmp2.y*=-1;
			tmp2.z*=-1;
		}
		if((angle_between(cam_to_eye_LES,tmp3) < -90)||(angle_between(cam_to_eye_LES,tmp3) > 90) )
		{
			tmp3.x*=-1;
			tmp3.y*=-1;
			tmp3.z*=-1;
		}		
		cam_to_eye.x=tmp1.x/3.0 + tmp2.x/3.0 + tmp3.x/3.0;
		cam_to_eye.y=tmp1.y/3.0 + tmp2.y/3.0 + tmp3.y/3.0;
		cam_to_eye.z=tmp1.z/3.0 + tmp2.z/3.0 + tmp3.z/3.0;
		
		CvPoint3D64f rotvector;//vector to rotate around
		CrossProduct(cam_to_eye_LES,cam_to_eye,rotvector);
		Normalize(rotvector);
		//calculate rotation matrix;
		double **rotation_mod = (double **)malloc(sizeof(double)*9);
		for(int k=0;k<3;k++)
		{
			rotation_mod[k]=(double*)malloc(sizeof(double)*3);
			memset(rotation_mod[k],0,3*sizeof(double));
		}

		RotationMatrix(rotvector,-angle_between(cam_to_eye_LES,cam_to_eye),rotation_mod);

		//update cameras rotation_matrix
		matrix_mul(rotation_mod,cam_it->rotation_apriori,cam_it->rotation);
		if (rotation_mod!=NULL)
		{
			for(int k=0;k<3;k++)
			{
				if (rotation_mod[k]!= NULL)free(rotation_mod[k]);
			}
			free(rotation_mod);
			rotation_mod = NULL;
		}
		i+=3;
	}
}


//  /target Thorsten
//  /applaud
#define TINY 1.0e-20

void ludcmp(double **a, int n, int *indx) 
{ 
	int i, imax, j, k; 
	float big, dum, sum, temp; 
	
	double vv[3]; // vv stores the implicit scaling of each row. 
	double d = 1.0; // No row interchanges yet. 
	for (i = 0; i < n; i++) 
	{ // Loop over rows to get the implicit scaling information
		big = 0.0; 
		for (j = 0; j < n; j++) 
		{
			if ((temp = fabs(a[i][j])) > big) big = temp; 
		}
		assert(big != 0.0);
		// No non-zero largest element. 
		vv[i] = 1.0 / big;// Save the scaling. 
	} 
	
	for (j = 0; j < n; j++) 
	{ // This is the loop over columns of Crouts method. 
		for (i = 0; i < j; i++) 
		{ // This is equation(2.3.12) except for i=j. 
			sum = a[i][j]; 
			for (k = 0; k < i; k++) sum -= a[i][k] * a[k][j]; 
			a[i][j] = sum; 
		} 
		big = 0.0; // Initialize for the search for largest pivot element. 
		for (i = j; i < n; i++) 
		{ // This is i=j of equation(2.3.12) and i = j + 1...N of equation(2.3.13)
			sum = a[i][j]; 
			for (k = 0; k < j; k++)
				sum -= a[i][k] * a[k][j]; 
			a[i][j] = sum; 
			if ((dum = vv[i] * fabs(sum)) >= big)
			{ 
				// Is the figure of merit for the pivot better than the best so far? 
				big = dum; 
				imax = i; 
			} 
		} 
		if (j != imax)
		{ // Do we need to interchange rows? 
			for (k = 0; k < n; k++) 
			{ // Yes, do so... 
				dum = a[imax][k]; 
				a[imax][k] = a[j][k]; 
				a[j][k] = dum; 
			} 
			d = -(d); // ... and change the parity of d. 
			vv[imax] = vv[j]; // Also interchange the scale factor. 
		} 
		indx[j] = imax; 
		if (a[j][j] == 0.0) a[j][j] = TINY; 
		// If the pivot element is zerothe matrixis singular (at least to the precision of the algorithm). For some applications on singular matrices, it is desirable to substitute TINY for zero. 
		if (j != (n-1)) 
		{ // Now, finally, divide by the pivot element. 
			dum = 1.0 / (a[j][j]); 
			for (i = j+1; i<n; i++) a[i][j] *= dum; 
		} 
	} // Go back for the next column in the reduction. 
}


void lubksb(double **a, int n, int *indx, double b[3]) 
{ 
	bool found = false;
	int i, ii, ip, j; 
	float sum; 
	for (i = 0; i < n; i++) 
	{ // When ii is set to a positive value, it will become the index of the first non vanishing element of b. We now do the forward substitution, equation(2.3.6). The only new wrinkle is to unscramble the permutation as we go. 
		ip = indx[i]; 
		sum = b[ip]; 
		b[ip] = b[i]; 
		if (found)
		{
			for (j = ii; j <= i-1; j++)
				sum -= a[i][j] * b[j]; 
		}
		else if (sum)
		{
			found = true;
			ii = i; // A non-zero element was encountered, so from now on we will have to do the sums in the loop above
		}
		b[i]=sum; 
	} 
	for (i = n-1; i >= 0; i--)
	{ // Now we do the back substitution, equation(2.3.7). 
		sum = b[i]; 
		for (j= i+1; j < n; j++)
			sum -= a[i][j] * b[j]; 
		b[i] = sum / a[i][i]; // Store a component of the solution vector X. 
	} // All done! 
}