#pragma once
#include "stdafx.h"
#include <vector>

#include "GTCamera.h"
#include "Table.h"

//calculates the determinant of a 3x3 matrix
double det(double **_mat_a);

//inverts _mat_a
bool invert_matrix(double **_mat_a,double **_invert);

void CrossProduct(CvPoint3D64f &_vec1,CvPoint3D64f &_vec2,CvPoint3D64f &_normal);
double ScalarProduct(CvPoint3D64f &_vec1,CvPoint3D64f &_vec2);

//returns the _point which is nearest( in a least error square sense) to all planes.
// returns false if det = 0 i.e. not solvable
bool point_nearest_to_planes(plane *_planes,int _numplanes, CvPoint3D64f &_point);


//calculates the plane from the given points on it
// and adds it to _planes
// see my thesis in section "calculate the cornea center" for details
void add_plane(GTCamera &_cam, CvPoint2D64f &_glint, CvPoint3D64f &_light, std::vector<plane> &_planes, int _store_at=-1);
void RotationMatrix(double x_angle,double y_angle,double z_angle,bool _invert_rotation, double** _matrix);
void RotationMatrix(CvPoint3D64f &_rotate_around,double _angle, double **_matrix);

//calculates a ray from the camera to the _glint
void unproject(GTCamera &_cam,CvPoint2D64f &_glint,CvPoint3D64f &_cam_to_glint);

//calculates the gaze direction(if all data is ready)

void calc_gaze( std::vector<GTCamera> &cams_,Table &_table);

template <typename T> void set_between(T& _toset,T _lower,T _upper)
{
	if(_toset < _lower) _toset =_lower;
	if(_toset > _upper) _toset =_upper;
}
double calculateQuality(std::vector<GTCamera> &_cams,Table &_table);

double random_flat(double _a, double _b);
double random_gauss(double _mean,double _dev);

void Normalize(CvPoint3D64f &_vec);
void matrix_mul(double **_A,double **_B, double **_out);

void adaptive_calibration(std::vector<GTCamera> &_cams,std::vector<plane> &_right_eye_planes);
double angle_between(CvPoint3D64f &_veca,CvPoint3D64f &_vecb);


void lubksb(double **a, int n, int *indx, double b[3]) ;
void ludcmp(double **a, int n, int *indx) ;