#pragma once
#include "stdafx.h"
#include "image_processing.h"
#include "highgui.h"
#include "algebra.h"
void invertImage(const IplImage *_src, IplImage *_dst)
{
	int  height    = _src->height;
	int width     = _src->width;
	int step      = _src->widthStep;
	int channels  = _src->nChannels;
	uchar *data      = (uchar *)_src->imageData;

	// invert the image
	for(int i=0;i<height;i++) for(int j=0;j<width;j++) for(int k=0;k<channels;k++)
		_dst->imageData[i*step+j*channels+k]=(uchar) 255-data[i*step+j*channels+k];

}
//we assume that all images are of the same type i.e. size,format... 
//dst = src1-src2 pixelwise and >=0
void subtract(const IplImage *_src1,const IplImage *_src2,IplImage *_dst)
{


	// because imageData is a signed char*
	unsigned char *data_dst=
		reinterpret_cast<unsigned char *>(_dst->imageData);
	unsigned char *data_src1=
		reinterpret_cast<unsigned char *>(_src1->imageData);
	unsigned char *data_src2=
		reinterpret_cast<unsigned char *>(_src2->imageData);

	//we assume 1 channel
	for (int i=0; i<_dst->width*_dst->height*_dst->nChannels; i++) 
	{
		data_dst[i] = (data_src1[i] > data_src2[i]) ? (data_src1[i] - data_src2[i]) : 0;

	}

}
// thresholds an image. every pixel smaller than the top X% brightness is set to 0
void adaptiveThreshold(IplImage *_src,IplImage *_dst,double _percentage)
{

	int width = _src->width;
	int height = _src->height;

	uchar* src_data = (uchar *) _src->imageData;	
	int pixelJump = _src->depth/8;
	IplImage *m_currentFrame = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 1);

	int src_step = 0;
	int dst_step = 0;

	//build histogram
	int hist[256];
	memset(hist,0,sizeof(int)*256);

	for (int y = 0; y < height; ++y)
	{
		for (int x = 0; x < width; ++x)
		{
			((uchar *)(m_currentFrame->imageData + dst_step))[x] = src_data[(src_step+x)*pixelJump];
			hist[src_data[(src_step+x)*pixelJump]]++;
		}
		dst_step += m_currentFrame->widthStep;
		src_step += width;
	}


	// calc threshold by histogram (get top _percentage brightness)
	int accum = 0;
	int binIndex = 255;

	while (accum < (1.0-_percentage) * width * height)
	{
		accum += hist[binIndex];
		binIndex--;
	}
	int m_threshold = binIndex;

	cvThreshold(m_currentFrame, _dst, m_threshold, 255, CV_THRESH_TOZERO);

	cvReleaseImage(&m_currentFrame);


}
//sets all pixels between min_percent and max_percent brightness to 0
void adaptiveThresholdBetween(IplImage *_src,IplImage *_dst,double min_percent, double max_percent)
{

	int width = _src->width;
	int height = _src->height;

	uchar* src_data = (uchar *) _src->imageData;	
	int pixelJump = _src->depth/8;
	IplImage *m_currentFrame = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 1);

	int src_step = 0;
	int dst_step = 0;

	//build histogram
	int hist[256];
	memset(hist,0,sizeof(int)*256);

	for (int y = 0; y < height; ++y)
	{
		for (int x = 0; x < width; ++x)
		{
			((uchar *)(m_currentFrame->imageData + dst_step))[x] = src_data[(src_step+x)*pixelJump];
			hist[src_data[(src_step+x)*pixelJump]]++;
		}
		dst_step += m_currentFrame->widthStep;
		src_step += width;
	}


	// calc threshold by histogram (get top _percentage brightness)
	int accum = 0;
	int binIndex = 0;

	int low_thresh,high_thresh;

	while (accum < ((min_percent) * (double) width * (double) height))
	{
		accum += hist[binIndex];
		binIndex++;
	}
	low_thresh = binIndex;
	binIndex =255;
	accum=0;
	while (accum < (1-max_percent) * width * height)
	{
		accum += hist[binIndex];
		binIndex--;
	}
	high_thresh = binIndex;

	// all brighter than high_thresh are set to 0
	cvThreshold(m_currentFrame, m_currentFrame, high_thresh, 255, CV_THRESH_TOZERO_INV);
	//all darker than low_thresh are set to 0
	cvThreshold(m_currentFrame, m_currentFrame, low_thresh, 255, CV_THRESH_TOZERO);

	cvCopyImage(m_currentFrame,_dst);
	cvReleaseImage(&m_currentFrame);


}

//compares 2 areas a,b
/* a < b ? -1 : a > b ? 1 : 0 */
static int __cmp_blobs( const void* _a, const void* _b, void* userdata )
{
	CvConnectedComp *blob_a = (CvConnectedComp*) _a;
	CvConnectedComp *blob_b = (CvConnectedComp*) _b;
	return (blob_a->area < blob_b->area ? -1 : blob_a->area > blob_b->area ? 1 : 0 );
}

//calculates a box around the pupil and the function itself returns the number of ellipses found
int find_pupil_pos(IplImage *_image,CvBox2D &_pupil)
{
	invertImage(_image,_image);		
#if POVRAYIMG
	adaptiveThreshold(_image,_image,0.999);//povray
#else
	adaptiveThreshold(_image,_image,0.98);
#endif
	CBlobResult blobs;
	CBlob ret;
	blobs = CBlobResult(_image,NULL,10,true);

	int min_blob_size = 100;

	//blobs.PrintBlobs("pupilblobs.txt");

	blobs.Filter(blobs, B_EXCLUDE, CBlobGetArea(), B_LESS, min_blob_size );
	blobs.Filter(blobs,B_EXCLUDE,CBlobGetExterior(),B_GREATER,0);
	blobs.Filter(blobs,B_EXCLUDE,CBlobGetParent(),B_GREATER,1);

	blobs.GetNthBlob(CBlobGetArea(),0,ret);

	_pupil.center.x = (ret.MinX() + ret.MaxX())/2.0;
	_pupil.center.y = (ret.MinY() + ret.MaxY())/2.0;
	_pupil.size.width = ret.MaxX() - ret.MinX();
	_pupil.size.height = ret.MaxY() - ret.MinY();
	
	////bounds checking
	set_between(_pupil.center.x,0.0f,(float)_image->width);
	set_between(_pupil.center.y,0.0f,(float)_image->height);
	set_between(_pupil.size.width,0.0f,(float)_image->width);
	set_between(_pupil.size.height,0.0f,(float)_image->height);

	return blobs.GetNumBlobs();
}
